unit uUserMainPrgm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, ComCtrls, AboutDlg, LibraryClass, uRegComp, uSearch,
  IAgaloLED;

type
  TInfoPriority = (ipNormal, ipSuccess, ipWarning, ipError);

  TUserMainForm = class(TForm)
    XDelContenaire: TPanel;
    BevelButton: TBevel;
    ButtonStartStop: TButton;
    ButtonOptions: TButton;
    ButtonQuit: TButton;
    ButtonAbout: TButton;
    ButtonUpdate: TButton;
    ViewAnimate: TAnimate;
    LabelInfo: TLabel;
    ProgressBar: TProgressBar;
    MemoInfo: TRichEdit;
    AboutDlg: TAboutDlg;
    RestartButton: TButton;
    ImageIcon: TImage;
    IAgaloLEDReady: TIAgaloLED;
    IAgaloLEDWork: TIAgaloLED;
    IAgaloLEDOther: TIAgaloLED;
    TimerLedYellow: TTimer;
    TimerStart: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ButtonAboutClick(Sender: TObject);
    procedure ButtonQuitClick(Sender: TObject);
    procedure RestartButtonClick(Sender: TObject);
    procedure ButtonStartStopClick(Sender: TObject);
    procedure ButtonOptionsClick(Sender: TObject);
    procedure TimerLedYellowTimer(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure TimerStartTimer(Sender: TObject);
  private
    { Dclarations prives }
    FLibClass: TXDLLib;
    FXDLReg  : TXDELRegistry;
    FViewDetail,
    FSEnabled: boolean;
    FLogString: TStringList;
    FTimeSearch: Cardinal;
    FTempSize  : Int64;
    FPersoSize : Int64;
    FLockXDel  : boolean;
    FSTStart   : boolean;
    procedure setLockXDel(Value: boolean);
    procedure setSEnabled(Value: boolean);
    procedure setViewDetail(Value: boolean);
    procedure setBlinkLed(Value: boolean);
    function  getBlinkLed: boolean;
    function AffichePourcentage(R: Real): boolean;
    procedure Refresh;
    procedure AfficheEnd;
  public
    { Dclarations publiques }
    procedure Recherche;
    procedure EndRecherche(EndStat: TCanStat);

    procedure AddInformationText(Text: string; const Priority: TInfoPriority = ipNormal);
    procedure InitializeWindows;
    procedure FinalizeWindows;
    property  StartEnabled: boolean read FSEnabled   write setSEnabled;
    property  ViewDetail  : boolean read FViewDetail write setViewDetail;
    property  LockXDel    : boolean read FLockXDel   write setLockXDel;
    property  IBlinkLed   : boolean read getBlinkLed write setBlinkLed;
  end;

var
  UserMainForm: TUserMainForm;
  StartNow    : boolean = false;

implementation

{$R *.DFM}

uses VersFile, OptionsDlg, uDiskUtils, uFileList;

resourcestring
  cStartBtn            = '&Dmarrer';
  cStopBtn             = '&Stop';
  cInitializeModule    = 'Initialisation du module de recherche ...';
  cInitializeSuccess   = 'Initialisation russie (Chargement n%d).';
  cInitializeFail      = 'Initialisation choue (%s).';
  cFinalizationModule  = 'Finalisation du module de recherche ...';
  cFinalizationSuccess = 'Finalisation russie.';
  cFinalizationFail    = 'Finalisation choue (%s).';

  rsFirstStart         = 'Afin de pouvoir effectuer un premier dmarrage, ' +
                         'il vous faut vrifier les options.';

  rsLibLocked          = 'Module verrouill par %s.';
  rsLibUnLock          = 'Module dverrouill.';

  rsCleanTrash         = 'Nettoyage de la corbeille ...';
  rsCleanTemp          = 'Nettoyage du dossier temporaire ...';
  rsCleanPerso         = 'Nettoyage du disque dur ...';
  rsWait               = 'En attente ...';

  rsFileSuppr          = ' * %s : OK';
  rsFileFailed         = ' * %s : FAILED';

  rsInterrupt          = 'Interruption utilisateur';
  rsTrashEmpty         = 'La corbeille a t nttoy.';
  rsTempEmpty          = 'Le dossier temporaire a t vid.';
  rsTempsSearch        = 'Temps mis pour la recherche        : %s';
  rsGainTemp           = 'Gain pour le dossier temporaire    : %s';
  rsGainSearch         = 'Gain pour la recherche personnelle : %s';
  rsGainTotal          = '--------------------------------------------------'#10 +
                         '                        Total      : %s';

function TUserMainForm.AffichePourcentage(R: Real): boolean;
begin
  result := not StartEnabled;
  ProgressBar.Max      := 1000;
  ProgressBar.Position := Round(R * 1000);
  LockXDel             := FLibClass.IsLocked;
  if (not FSTStart) and (not LockXDel) and (FLibClass.SearchStat = []) then
  begin
    LabelInfo.Caption := rsWait;
    StartEnabled      := true;
  end;
  Application.ProcessMessages;
end;

procedure TUserMainForm.AddInformationText(Text: string; const Priority: TInfoPriority);
var lg : TLogEvent;
begin
  lg := TLogEvent(FXDLReg.Journal.LogEvent);
  ActiveControl := MemoInfo;
  case Priority of
    ipNormal : begin
                 MemoInfo.SelAttributes.Color := clSilver;
                 if leDeleteFile in lg then FLogString.Add(Text);
               end;
    ipSuccess: begin
                 MemoInfo.SelAttributes.Color := clLime;
                 if leDeleteFile in lg then FLogString.Add(Text);
               end;
    ipWarning: begin
                 MemoInfo.SelAttributes.Color := clYellow;
                 if leWarning in lg then FLogString.Add(Text);
               end;
    ipError  : begin
                 MemoInfo.SelAttributes.Color := clRed;
                 if leError in lg then FLogString.Add(Text);
               end;
  end;
  MemoInfo.Lines.Add(Text);
  MemoInfo.SelAttributes       := MemoInfo.DefAttributes;
end;

procedure TUserMainForm.InitializeWindows;
begin
  AddInformationText(HeadGenerator, ipNormal);
  try
    AddInformationText(cInitializeModule, ipNormal);
    FLibClass := TXDLLib.Create(XDELWINID, GetRessourceFile(Application.ExeName, viProducteName), XDELWINKEY);
    FLibClass.PourcentageProc := AffichePourcentage;
    FLibClass.RefreshTime     := FXDLReg.Affichage.Refresh;
    FLibClass.EndProc         := EndRecherche;
    AddInformationText(Format(cInitializeSuccess, [FLibClass.NChargement]), ipSuccess);
    AddInformationText('', ipNormal);
    FLibClass.Lock;
    IAgaloLEDReady.LedOn := true;
  except
    on e: Exception do
      AddInformationText(Format(cInitializeFail, [e.Message]), ipError);
  end;
end;

procedure TUserMainForm.FinalizeWindows;
begin
  try
    IAgaloLEDReady.LedOn := false;
    ProgressBar.Position := 0;
    if Assigned(FLibClass) then
    begin
      // On dbloque pour les autres application
      FLibClass.UnLock;
      AddInformationText(cFinalizationModule, ipNormal);
      if not Assigned(FLibClass) then raise EXDLLib.Create('Erreur ...');
      FLibClass.Free; FLibClass := nil;
      AddInformationText(cFinalizationSuccess, ipSuccess);
      AddInformationText('', ipNormal);
    end;
  except
    on e: Exception do
      AddInformationText(Format(cFinalizationFail, [e.Message]), ipError);
  end;
end;

procedure TUserMainForm.FormCreate(Sender: TObject);
begin
  FXDLReg := TXDELRegistry.Create;
  FXDLReg.LoadFromRegistry;

  FLogString := TStringList.Create;

  FSEnabled   := true;
  FSTStart    := false;
  InitializeWindows;
  Refresh;
end;

procedure TUserMainForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  if Assigned(FLibClass) and (FLibClass.SearchStat <> []) and (not FLibClass.isLocked) then Action := caNone else
  begin
    FinalizeWindows;
    MemoInfo.Refresh;

    if (Left <> FXDLReg.Affichage.XPos) or (Top <> FXDLReg.Affichage.YPos) then
    begin
      FXDLReg.Affichage.XPos := Left;
      FXDLReg.Affichage.YPos := Top;
      FXDLReg.SaveToRegistry;
    end;

    FXDLReg.Free;
    FLogString.Free;
  end;
end;

procedure TUserMainForm.ButtonAboutClick(Sender: TObject);
begin
  IBlinkLed := true;
  AboutDlg.Execute;
  IBlinkLed := false;
end;

procedure TUserMainForm.ButtonQuitClick(Sender: TObject);
begin
  Close;
end;

procedure TUserMainForm.setBlinkLed(Value: boolean);
begin
  TimerLedYellow.Enabled := Value;
  if not Value then
    IAgaloLEDOther.LedOn := false;
end;

function TUserMainForm.getBlinkLed: boolean;
begin
  result := TimerLedYellow.Enabled;
end;

procedure TUserMainForm.setLockXDel(Value: boolean);
begin
  if Value <> FLockXDel then
  begin
    case Value of
      true : begin
               AddInformationText(Format(rsLibLocked, [FLibClass.LockedID]), ipWarning);
               ButtonStartStop.Enabled := false;
               ButtonOptions.Enabled   := false;
             end;
      false: begin
               AddInformationText(rsLibUnLock, ipSuccess);
               ButtonStartStop.Enabled := true;
               ButtonOptions.Enabled   := true;
               // On bloque pour les autres application
               FLibClass.Lock;
             end;
    end;
    FLockXDel := Value;
    AddInformationText('');
  end;
end;

procedure TUserMainForm.setSEnabled(Value: boolean);
begin
  if Value <> FSEnabled then
  begin
    case Value of
      true : begin
               ButtonStartStop.Caption := cStartBtn;
               Screen.Cursor           := crDefault;
               ButtonOptions.Enabled   := true;
               RestartButton.Enabled   := true;
               if FXDLReg.Affichage.Minimize then Application.Restore;
               IAgaloLEDWork.LedOn     := false;
             end;
      false: begin
               IAgaloLEDWork.LedOn     := true;
               ButtonStartStop.Caption := cStopBtn;
               Screen.Cursor           := crHourGlass;
               ButtonOptions.Enabled   := false;
               RestartButton.Enabled   := false;
               if FXDLReg.Affichage.Minimize then Application.Minimize;
             end;
    end;
    FSEnabled := Value;
  end;
end;

procedure TUserMainForm.setViewDetail(Value: boolean);
begin
  case Value of
    true : ClientHeight := 2 * XDelContenaire.Height;
    false: ClientHeight :=     XDelContenaire.Height;
  end;
  FViewDetail := Value;
end;

procedure TUserMainForm.RestartButtonClick(Sender: TObject);
begin
  FinalizeWindows;
  Application.ProcessMessages;
  InitializeWindows;
end;

procedure TUserMainForm.Recherche;
begin
  FTimeSearch:= GetTickCount; FLogString.Clear;  FTempSize := -1;  FPersoSize := -1; FSTStart := true;
  if not Assigned(FLibClass) then Exit;
  try
    if (FXDLReg.Nettoyage.CleanTrash) and (not StartEnabled) then
    begin
      LabelInfo.Caption := rsCleanTrash;
      ViewAnimate.CommonAVI := aviEmptyRecycle;
      ViewAnimate.Active    := true;
      Application.ProcessMessages;
      FLibClass.CleanTheTrash;
    end;
    if (FXDLReg.Nettoyage.CleanTemp) and (not StartEnabled) then
    begin
      LabelInfo.Caption := rsCleanTemp;
      ViewAnimate.CommonAVI := aviDeleteFile;
      ViewAnimate.Active    := true;
      Application.ProcessMessages;
      FTempSize  := FLibClass.CleanTemp;
    end;
    if (FXDLReg.Nettoyage.CleanPerso) and (not StartEnabled) then
    begin
      LabelInfo.Caption := rsCleanPerso;
      ViewAnimate.CommonAVI := aviFindFile;
      ViewAnimate.Active    := true;
      Application.ProcessMessages;
      FLibClass.InitializeTree(FXDLReg.Arbre.FStream);
      FLibClass.CleanPerso;
    end;
  except
    FSTStart := false;
    raise;
  end;
  FSTStart := false;
end;

procedure TUserMainForm.AfficheEnd;
var le: TLogEvent;
begin
  le := TLogEvent(FXDLReg.Journal.LogEvent);
  if (leTimeSearch in le) and (FTimeSearch <> 0) then
    AddInformationText(
      Format(rsTempsSearch, [TimToStr(GetTickCount - FTimeSearch)]));
  if (leSizeFree in le) then
  begin
    if FTempSize <> -1 then
      AddInformationText(Format(rsGainTemp, [SizeToStr(FTempSize)]));
    if FPersoSize <> -1 then
      AddInformationText(Format(rsGainSearch, [SizeToStr(FPersoSize)]));
    if (FTempSize <> -1) and (FPersoSize <> -1) then
      AddInformationText(Format(rsGainTotal, [SizeToStr(FTempSize + FPersoSize)]));
  end;
  if FXDLReg.Journal.JournalActive then
    FLibClass.WriteToLogFile(
      FXDLReg.Journal.DirName + rsLogFileName,
      FLogString.Text,
      FXDLReg.Journal.EraseFile);
  ViewAnimate.CommonAVI := aviNone;
  TimerStart.Enabled := true;
end;

procedure TUserMainForm.EndRecherche(EndStat: TCanStat);
var TxtAdd   : TStringlist;
    I, FileSz: Integer;
begin
  case EndStat of
    CleanTrash   : AddInformationText(rsTrashEmpty);
    CleanTempDir : AddInformationText(rsTempEmpty);
    CleanPerso   : begin
                     AddInformationText('', ipNormal);
                     TxtAdd := TStringList.Create;
                     try
(* Recherche *)
                       FPersoSize       := 0;
                       TxtAdd.CommaText := FLibClass.FileList;
                       if TxtAdd.CommaText <> '' then
                       begin
                         if FXDLReg.Nettoyage.Confirm then
                         begin
                           Application.Restore;
                           ModifyFileList(TxtAdd);
                         end;

                         for I := 0 to TxtAdd.Count - 1 do
                         begin
                           if not FXDLReg.Nettoyage.Test then
                           begin
                             FileSz := FLibClass.DeleteFile(TxtAdd[I],
                               FXDLReg.Nettoyage.Send2Trash,
                               FXDLReg.Nettoyage.ROHConfirm = 0)
                           end else
                             FileSz := 0;
                           if fileSz <> 0 then
                             AddInformationText(Format(rsFileSuppr, [TxtAdd[I]]))
                           else
                             AddInformationText(Format(rsFileFailed, [TxtAdd[I]]), ipWarning);
                           FPersoSize := FPersoSize + FileSz;
                         end;
                       end;
(* Warning *)
                       AddInformationText('', ipNormal);
                       TxtAdd.CommaText := FLibClass.WarningList;
                       if TxtAdd.CommaText <> '' then
                       AddInformationText(TxtAdd.Text, ipWarning);
(* Erreur *)
                       TxtAdd.CommaText := FLibClass.ErrorList;
                       if TxtAdd.CommaText <> '' then
                       AddInformationText(TxtAdd.Text, ipError);
                     finally
                       TxtAdd.Free;
                     end;
                     Application.ProcessMessages;
                     AfficheEnd;
                   end;
  end;
end;

procedure TUserMainForm.ButtonStartStopClick(Sender: TObject);
begin
  if FXDLReg.FirstStart then
  begin
    MessageDlg(rsFirstStart, mtWarning, [mbOk], 0);
    ButtonOptions.Click;
  end;
  if StartEnabled then
  begin
    StartEnabled := false;
    try
      // Start search
      Recherche;
    except
      on e: Exception do
      begin
        AddInformationText(e.Message, ipError);
      end;
    end;
  end else
  begin
    FLibClass.Abort;
    AddInformationText(rsInterrupt, ipError);
  end;
end;

procedure TUserMainForm.ButtonOptionsClick(Sender: TObject);
begin
  IBlinkLed := true;
  if OpenOptionsDlg(Handle) then
  begin
    FXDLReg.LoadFromRegistry;
    Refresh;
  end;
  IBlinkLed := false;
end;

procedure TUserMainForm.Refresh;
begin
  ViewDetail := FXDLReg.Affichage.Detail;
  if Assigned(FLibClass) then
    FLibClass.RefreshTime := FXDLReg.Affichage.Refresh;
end;

procedure TUserMainForm.TimerLedYellowTimer(Sender: TObject);
begin
  IAgaloLEDOther.LedOn := not IAgaloLEDOther.LedOn;
end;

procedure TUserMainForm.FormShow(Sender: TObject);
begin
  if (FXDLReg.Affichage.XPos <> -1) and (FXDLReg.Affichage.YPos <> -1) then
  begin
    Left := FXDLReg.Affichage.XPos;
    Top  := FXDLReg.Affichage.YPos;
  end else
  begin
    Left := (Screen.Width  - Width ) div 2;
    Top  := (Screen.Height - Height) div 2;
  end;
end;

procedure TUserMainForm.TimerStartTimer(Sender: TObject);
begin
  TimerStart.Enabled := false;
  case TimerStart.Tag of
    0 : if StartNow then ButtonStartStop.Click;
    1 : if StartNow then Close;
  end;
  TimerStart.Tag := TimerStart.Tag + 1;
end;

end.
