unit Algo_Rech;

interface

uses
  Windows, Classes, SysUtils, Algo, uSearch;

type

{ En cas d'erreur }

  ERAException = class(EAbort);

{ TARFileList }

{ Liste de fichier spcifi dans le paramtre du constructeur, le programme
  fait une recherche pour un chemin et un masque de fichier }

  TARFileList = class
  private
    FPath: string;
    FDir : Boolean;
    FList: TStringList;
  protected
    function getItem(Index: Integer) : string; virtual;
    function getCount: integer; virtual;
  public
    constructor Create(const Path: String; Dir: boolean);
    destructor Destroy; override;

    function Execute: TStrings; virtual;

    property Items[Index : Integer]: string read getItem; default;
    property Count: Integer read getCount;
  end;

{ TRechercheAdapter }

{ Implementation de la recherche }

  TRechercheAdapter = class
  private
    FCurrentTree: IArborescense;
    FCurrentMask: IMasque;
    FMasquesList: TMasqueList;
    FMasqueCount: Integer;
    FPercentProc: TRealPercentProc;
    FPercent    : Real;
  protected
    function getPercent: Real;
    procedure CallPercentProc(Pos: Integer);
    function getRecursiveMasquesList: TMasqueList; virtual;
    procedure ChangeIfNecesary; virtual;
    function getPercentProc: TRealPercentProc; virtual;
    procedure setPercentProc(Value: TRealPercentProc); virtual;
  public
    constructor Create(CurrentTree: IArborescense; CurrentMask: IMasque);
    destructor Destroy; override;
    function Execute: TStrings;
    property RecursiveMasquesList: TMasqueList read getRecursiveMasquesList;
    property PercentProc: TRealPercentProc read getPercentProc write setPercentProc;
  end;

{ IRechArborescense }

{ Represente un arbre de recherche }

  IRechArborescense = interface(IArborescense)
    ['{1181CC76-BFC3-43A6-B74E-EA29BD0EF120}']
    // Protected
    function getPercent: Real;
    function getRecursiveMasquesList: TMasqueList;
    function getPercentProc: TRealPercentProc;
    procedure setPercentProc(Value: TRealPercentProc);

    // Public
    function Execute: TStrings;
    property RecursiveMasquesList: TMasqueList read getRecursiveMasquesList;
    property PercentProc: TRealPercentProc read getPercentProc write setPercentProc;
  end;

{ TRAPosteDeTravail }

{ Represente un poste de travail de Recherche }

  TRAPosteDeTravail = class(TPosteDeTravail, IArborescense, IRechArborescense)
  private
    FImplementeClass: TRechercheAdapter;
  public
    constructor Create(AllocSize: Integer);
    destructor Destroy; override;
    property ImplementeClass: TRechercheAdapter read FImplementeClass implements IRechArborescense;

    function Add(Name: string): integer; override;
  end;

{ TRADisque }

{ Represente un disque pour la recherche }

  TRADisque = class(TDisque, IArborescense, IMasque, IRechArborescense)
  private
    FImplementeClass: TRechercheAdapter;
  public
    constructor Create(Name: char; Parent: IArborescense; AllocSize, AllocMask: Integer);
    destructor Destroy; override;
    property ImplementeClass: TRechercheAdapter read FImplementeClass implements IRechArborescense;

    function Add(Name: string): integer; override;
  end;

{ TRADossier }

{ Represente un dossier pour la recherche }

  TRADossier = class(TDossier, IArborescense, IMasque, IRechArborescense)
  private
    FImplementeClass: TRechercheAdapter;
  public
    constructor Create(Name: string; Parent: IArborescense; AllocSize, AllocMask: Integer);
    destructor Destroy; override;
    property ImplementeClass: TRechercheAdapter read FImplementeClass implements IRechArborescense;

    function Add(Name: string): integer; override;
  end;

implementation

uses uDiskUtils;

{ Utility }

function ExtractPathNameOf(P: String) : string;
begin
  result := P;
  if(result[Length(result)] = '\') then setLength(result, length(result) - 1);
  result := ExtractFileName(result);
end;

{ TARFileList }

constructor TARFileList.Create(const Path: String; Dir: boolean);
begin
  FPath := Path;
  FDir  := Dir;
  FList := TStringList.Create;
end;

destructor TARFileList.Destroy;
begin
  FList.Free;
end;

function TARFileList.getItem(Index: Integer) : string;
begin
  result := FList.Strings[Index];
end;

function TARFileList.getCount: integer;
begin
  result := FList.Count;
end;

function TARFileList.Execute: TStrings;
var Rec: TSearchRec;
    R  : Integer;
begin
  FList.Clear;
  if FDir then
    R := FindFirst(FPath, faDirectory, Rec)
  else
    R := FindFirst(FPath, faAnyFile - faVolumeID - faDirectory, Rec);
  try
    while R = 0 do
    begin
      if (Rec.Name <> '.') and (Rec.Name <> '..') then
      if (not FDir) or (FDir and (Rec.Attr and faDirectory > 0)) then
      FList.Add(ExtractFilePath(FPath) + Rec.Name);
      R := FindNext(Rec);
    end;
  finally
    FindClose(Rec);
  end;
  result := TStringList.Create;
  result.Assign(FList);
end;

{ TRechercheAdapter }

constructor TRechercheAdapter.Create(CurrentTree: IArborescense; CurrentMask: IMasque);
begin
  FCurrentTree := CurrentTree;
  FCurrentMask := CurrentMask;
  FMasquesList := TMasqueList.Create(16);
end;

destructor TRechercheAdapter.Destroy;
begin
  FMasquesList.Free;
  inherited Destroy;
end;

function TRechercheAdapter.getPercentProc: TRealPercentProc;
begin
  if not Assigned(FPercentProc) then
    FPercentProc := (FCurrentTree.Parent as IRechArborescense).PercentProc;
  result := FPercentProc;
end;

procedure TRechercheAdapter.setPercentProc(Value: TRealPercentProc);
begin
  if @value <> @FPercentProc then
    FPercentProc := Value;
end;

function TRechercheAdapter.getPercent: Real;
begin
  result := FPercent;
  if (result = 0) and Assigned(FCurrentTree.Parent) then
    result := (FCurrentTree.Parent as IRechArborescense).getPercent;
end;

procedure TRechercheAdapter.CallPercentProc(Pos: Integer);
  function getFactor(Tree: IArborescense): Int64;
  begin
    result := 1;
    if Assigned(Tree) then
      result := Tree.Count * getFactor(Tree.parent);
  end;
var Factor : Int64;
    FPerc  : TRealPercentProc;
    ParPerc: Real;
begin
  FPerc := PercentProc;
  if Assigned(FPerc) then
  begin
    Factor   := getFactor(FCurrentTree);
    FPercent := Pos / Factor;
    ParPerc  := 0;
    if Assigned(FCurrentTree.parent) then
      ParPerc := (FCurrentTree.parent as IRechArborescense).getPercent;
    FPercent  := ParPerc + FPercent;
    if not FPerc(FPercent) then
      raise ERAException.Create('User abort');
  end;
end;

function TRechercheAdapter.Execute: TStrings;
var RechString: TStrings;
    I         : Integer;
begin
  result := TStringList.Create;
  // Recherche des rperoires et addition de ceux non existant
  if (FCurrentTree as IRechArborescense).RecursiveMasquesList.Count > 0 then
  begin
    RechString := TARFileList.Create(FCurrentTree.path + '*.*', true).Execute;
    try
      for I := 0 to RechString.Count - 1 do
        if FCurrentTree.IndexOf(ExtractPathNameOf(RechString[I])) = -1 then
          FCurrentTree.Add(ExtractPathNameOf(RechString[I]));
    finally
      RechString.Free;
    end;
  end;

  // Recherche dans les dossiers
  for I := 0 to FCurrentTree.Count - 1 do
  begin
    RechString := (FCurrentTree.Items[I] as IRechArborescense).Execute;
    try
      result.AddStrings(RechString);
    finally
      RechString.Free;
    end;
    CallPercentProc(I + 1);
  end;

  // Recherche des fichiers
  if Assigned(FCurrentMask) then
  with (FCurrentTree as IRechArborescense) do
    for I := 0 to RecursiveMasquesList.Count - 1 do
    begin
      RechString := TARFileList.Create(FCurrentTree.path + RecursiveMasquesList[I].masque, false).Execute;
      try
        result.AddStrings(RechString);
      finally
        RechString.Free;
      end;
    end;
end;

procedure TRechercheAdapter.ChangeIfNecesary;
var C, I, Index: Integer;
    RArbre     : IRechArborescense;
begin
  if Assigned(FCurrentTree.parent) and Assigned(FCurrentMask) then
  begin
    C := FCurrentMask.Masques.Count + (FCurrentTree.parent as IRechArborescense).RecursiveMasquesList.Count;
    if C <> FMasqueCount then
    begin
      FMasquesList.Clear;
      with FCurrentMask do
        For I := 0 to Masques.Count - 1 do
          if FMasquesList.IndexOf(Masques[I]) = -1 then
            FMasquesList.Add(Masques[I]);
      RArbre := FCurrentTree.parent as IRechArborescense;
      For I := 0 to RArbre.RecursiveMasquesList.Count - 1 do
        if RArbre.RecursiveMasquesList[I].recurse then
        begin
          Index := FMasquesList.IndexOf(RArbre.RecursiveMasquesList[I]);
          if Index = -1 then
            FMasquesList.Add(RArbre.RecursiveMasquesList[I])
          else
            FMasquesList[Index].recurse := true;
        end;
      FMasqueCount := C;
    end;
  end else
    if Assigned(FCurrentMask) then
      with FCurrentMask do
        for I := 0 to Masques.Count - 1 do
          if FMasquesList.IndexOf(Masques[I]) > -1 then
            FMasquesList.Add(Masques[I])
    else
      FMasquesList.Clear;
end;

function TRechercheAdapter.getRecursiveMasquesList: TMasqueList;
begin
  ChangeIfNecesary;
  result := FMasquesList;
end;

{ TRAPosteDeTravail }

constructor TRAPosteDeTravail.Create(AllocSize: Integer);
begin
  inherited Create(AllocSize);
  FImplementeClass := TRechercheAdapter.Create(Self, nil);
end;

destructor TRAPosteDeTravail.Destroy;
begin
  FImplementeClass.Free;
  inherited Destroy;
end;

function TRAPosteDeTravail.Add(Name: string): integer;
begin
  Result := Add(TRADisque.Create(Name[1], Self, 32, 16));
end;

{ TRADisque }

constructor TRADisque.Create(Name: char; Parent: IArborescense; AllocSize, AllocMask: Integer);
begin
  inherited Create(Name, Parent, AllocSize, AllocSize);
  FImplementeClass := TRechercheAdapter.Create(Self, Self);
end;

destructor TRADisque.Destroy;
begin
  FImplementeClass.Free;
  inherited Destroy;
end;

function TRADisque.Add(Name: string): integer;
begin
  Result := Add(TRADossier.Create(Name, Self, 32, 16));
end;

{ TRADossier }

constructor TRADossier.Create(Name: string; Parent: IArborescense; AllocSize, AllocMask: Integer);
begin
  inherited Create(Name, Parent, AllocSize, AllocSize);
  FImplementeClass := TRechercheAdapter.Create(Self, Self);
end;

destructor TRADossier.Destroy;
begin
  FImplementeClass.Free;
  inherited Destroy;
end;

function TRADossier.Add(Name: string): integer;
begin
  Result := Add(TRADossier.Create(Name, Self, 32, 16));
end;

end.
