unit dllClasses;

interface

uses
  windows, SysUtils, Controls, Registry, uLogiciel, classes,
  ShareMemUtils, uSHFileOp, uDiskUtils, uWindows, uSearch, Algo,
  Algo_Rech, Thread_Algo;

type

{ TXDLLib }

{ Represente la librairie et regroupe ainsi toute ces fonctions. }

  TXDLLib        = class(TThread)
  private
    FSharedMem    : TSharedMem;
    FMutex        : TMutex;
    FNbLoaded     : PInteger;
    FRecOperation : PRecOperation;
    FChangeProc   : TChangeEvent;
    FPDTravail    : TPosteDeTravail;
    FThreadSearch : TSearchThread;
    FMasqueList   : TList;
    FSearchCont   : ^boolean;
  protected
    procedure ChangeOpState(s: TOperation; add: boolean); virtual;
    function  getLoaded : Integer; virtual;
    function  getOperationInfo : TRecOperation; virtual;
    function  RecherchePourcentageProc(Pourc:Real): boolean; virtual;

    procedure Execute; override;
  public
    constructor Create;
    destructor Destroy; override;

    function ClearWindowsTrash: boolean;
    function ClearWindowsTemp : Int64;

    procedure StopSearch;
    procedure StartSeatch;

    procedure InitializeTree;

    property OperationInfo: TRecOperation read getOperationInfo;
    property LoadedNumber : Integer       read getLoaded;
    property OnChangeProc : TChangeEvent  read FChangeProc write FChangeProc;
  end;

{ Variables representant la librairie }
var Librairie: TXDLLib;

implementation

const
  XDELSHARE = 'XDEL_WIN4_SEARCH';
  XDELMUTEX = 'XDEL_WIN4_MUTEX';
  TIMEOUT   = 2000;

{ TXDllib }

constructor TXDLLib.Create; // Initialization
var ShMemSize: Integer;
begin
  FChangeProc  := nil;
  // On crer un Mutex
  FMutex       := TMutex.Create(XDELMUTEX);
  if FMutex.Get(TIMEOUT) then
  begin
    // Aprs protection, on crer la partie de mmoire partager
    ShMemSize    := SizeOf(Integer) + SizeOf(Boolean) + SizeOf(TRecOperation);
    FSharedMem   := TSharedMem.Create(XDELSHARE, ShMemSize);

    // Nombre de fois que la librairie a t charger
    FNbLoaded              := FSharedMem.Buffer;
    // Annulation de la recherche
    Integer(FSearchCont)   := Integer(FNbLoaded)     + SizeOf(Integer);
    // Tableau contenant des informations tel que quel est le travail en cours
    Integer(FRecOperation) := Integer(FSearchCont)   + SizeOf(Boolean);
    // La Librairie est lu une fois de plus ...
    Inc(FNbLoaded^);
    // On librre l'accs
    FMutex.Release;
  end;
  // Pas de recherche en cours
  FThreadSearch := nil;
  // Creer un poste de travail (n'est pas en partage)
  FPDTravail    := TPosteDeTravail.Create(26);
  // Creer un liste permettant de lister tous les masques possible.
  FMasqueList   := TList.Create;
  // Initialize l'arbre  partir des donnes dans la base de registre
  InitializeTree;
  inherited Create(false);
  Priority := tpIdle;
end;

destructor TXDLLib.Destroy;
var I: Integer;
begin
  // Si on quite : vaut mieux quitter la recherche
  if Assigned(FThreadSearch) then
    FThreadSearch.Terminate;
  // Libre tous les masques
  for I := 0 to FMasqueList.Count - 1 do TMasque(FMasqueList[I]).Free;
  // Libre l'arborescense
  FPDTravail.Free;
  // Demande la main
  if FMutex.Get(TIMEOUT) then
  begin
    // La libriarie est libr
    Dec(FNbLoaded^);
    FRecOperation := nil;
    // On libre la mmoire partager
    FSharedMem.Free;
    // On relache le Mutex
    FMutex.Release;
  end;
  // On librre le mutex
  FMutex.Free;
  inherited Destroy;
end;

procedure TXDLLib.Execute;
begin
  while not Terminated do
  begin

  end;
end;

function TXDLLib.getLoaded : Integer;
begin
  // On retourne le nombrte de fois que la librairie est lu
  result := FNbLoaded^;
end;

function TXDLLib.getOperationInfo : TRecOperation;
begin
  // Information sur les oprations
  result := FRecOperation^;
end;

function TXDLLib.ClearWindowsTrash: boolean;
begin
  // On quitte si on vide dj la corbeille
  if ClearTrash in FRecOperation.Operations  then
  begin
    result := false;
    exit;
  end;
  // On vide la corbeille
  ChangeOpState(ClearTrash, true);
  try
    ShellEmptyRecycleBin;
    result := true;
  finally
    ChangeOpState(ClearTrash, false);
  end;
end;

function TXDLLib.ClearWindowsTemp : Int64;
var
  ShellStruct: TShellFileOpStruct;
  TmpDir     : string;
begin
  // On quite si on vide dj le dossier temporaire
  if ClearTemp in FRecOperation.Operations then
  begin
    result := -1;
    exit;
  end;
  // On nettoie le dossier temporaire
  ChangeOpState(ClearTemp, true);
  try
    TmpDir             := UpperCase (NormalDir(GetTempDir));
    result             := SizeOfDir(TmpDir);
    ShellStruct.wFunc  := foDelete;
    ShellStruct.pFrom  := TmpDir + '*.*';
    ShellStruct.fFlags := [fofNoConfirmation, fofSilent];

    ShellFileOperation(ShellStruct);
    result := result - SizeOfDir(TmpDir);
  finally
    ChangeOpState(ClearTemp, false);
  end;
end;

{ Utility }

procedure TXDLLib.ChangeOpState(s: TOperation; add: boolean);
begin
  // Change le status d'une opration
  if FMutex.Get(TIMEOUT) then
  begin
    if add then FRecOperation.Operations := FRecOperation.Operations + [s]
           else FRecOperation.Operations := FRecOperation.Operations - [s];
    FMutex.Release;
    if Assigned(FChangeProc) then FChangeProc;
  end;
end;

procedure TXDLLib.InitializeTree;
var Reg       : TRegistry;
    DataStr   : TMemoryStream;
    BinSize, I: Integer;
begin
  // On lit les donnes sur l'arbre dans la base de registre
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.OpenKey(GetRegKeyLogi(XDEL4_COMPANY), true);
    try
      BinSize := Reg.GetDataSize(cTreeValue);
      DataStr := TMemoryStream.Create;
      if BinSize > 0 then
      begin
        DataStr.SetSize(BinSize);
        Reg.ReadBinaryData(cTreeValue, DataStr.Memory^, BinSize);
        DataStr.Position := 0;
        FPDTravail.LoadFromStream(DataStr);
      end;
    finally
      Reg.CloseKey;
    end;
  finally
    Reg.Free;
  end;
  // On met  jour les informations sur les masques
  for I := 0 to FMasqueList.Count - 1 do TMasque(FMasqueList[I]).Free;
  FMasqueList.Clear;
  ChangeArborescenseMasque(FPDTravail, FMasqueList);
end;

procedure TXDLLib.StopSearch;
begin
  if FMutex.Get(TIMEOUT) then
  begin
    FSearchCont^ := false;
    FMutex.Release;
  end;
end;

procedure TXDLLib.StartSeatch;
var p : TRAPosteDeTravail;
begin
  // On quite si on vide dj le dossier temporaire
  if SearchFile in FRecOperation.Operations then exit;
  // On cherche
  ChangeOpState(SearchFile, true);
  if FMutex.Get(TIMEOUT) then
  begin
    FSearchCont^ := true;
    FMutex.Release;
  end;
  p := TRAPosteDeTravail.Create(26);
  try
    p.Assign(FPDTravail);
    FThreadSearch := TSearchThread.Create(p, tpNormal, TStringList.Create());
    FThreadSearch.PercentProc := RecherchePourcentageProc;
    FThreadSearch.Resume;
    FThreadSearch.WaitFor;
  finally
    p.Free;
    FThreadSearch := nil;
    ChangeOpState(SearchFile, false);
  end;
end;

function TXDLLib.RecherchePourcentageProc(Pourc:Real): boolean;
begin
  result := true;
  if FMutex.Get(TIMEOUT) then
  begin
    result                    := FSearchCont^;
    FRecOperation.Pourcentage := Pourc;
    FMutex.Release;
  end;
end;

initialization
  Librairie := TXDLLib.Create;

finalization
  Librairie.Free;

end.
