unit DLLGest;

interface

uses GlobClass, uSearch;

// Recherche

procedure InitLibrarie(ID, lParam, hParam: PChar);
procedure FreeLibrarie();

function  GetNbLibrairy(): Integer;
function  LengthAppNames(): Integer;
procedure GetAppNames(Data: PChar);

procedure setEndProc(Value: TEndSearchProc); stdcall;
function GetCurrentPercent: Real;
function GetCurrentStatus : TCanStats;

function  InitPersoTree(Data: Pointer; Size: Integer): boolean;
function  CleanWindows(what: Integer): Integer;
procedure getSearchLists(what: Integer; Data: PChar);
function  DeleteFile(const FileName: string; toTrash: boolean; deleteIfRH: boolean): integer;

// Bloque l'utilisation de la librairie

function LockLib(ID: PChar): boolean;
function UnLockLib(ID: PChar): boolean;
function IsLocked: boolean;
procedure NameLocked(ID: PChar);

// Log file
procedure CreateFileLog(FileName, ID, Data: PChar; Erase: boolean);

implementation

uses Classes, SysUtils, uDiskUtils, uSHFileOp;

(* Librairie - Recherche *)

var
  Librairie: TXDLLib;

procedure InitLibrarie(ID, lParam, hParam: PChar);
begin
  Librairie := TXDLLib.Create(ID, lparam, hparam);
end;

procedure FreeLibrarie();
begin
  if Assigned(Librairie) then
  begin
    Librairie.Free;
    Librairie := nil;
  end;
end;

procedure setEndProc(Value: TEndSearchProc);
begin
  Librairie.EndProc := Value;
end;

function  GetNbLibrairy(): Integer;
begin
  result := Librairie.LoadedNum;
end;

function  LengthAppNames(): Integer;
begin
  result := length(Librairie.AppNames);
end;

procedure GetAppNames(Data: PChar);
begin
  StrPCopy(Data, Librairie.AppNames);
end;

function  InitPersoTree(Data: Pointer; Size: Integer): boolean;
begin
  result := Librairie.InitializeTree(Data, Size);
end;

function  CleanWindows(what: Integer): Integer;
begin
  result := 0;
  case what of
    CLEAN_TRASH_PROC : Librairie.CleanWindowsTrash;
    CLEAN_TEMP_PROC  : result := Librairie.CleanWindowsTempDirectory;
    CLEAN_PERSO_PROC : Librairie.SearchFile(tpNormal);
    CLEAN_ABORT_PROC : Librairie.Abort := true;
  end;
end;

procedure getSearchLists(what: Integer; Data: PChar);
begin
  case what of
    LIST_FILE           : StrPCopy(Data, Librairie.FileList);
    LIST_WARNING        : StrPCopy(Data, Librairie.Warning);
    LIST_ERROR          : StrPCopy(Data, Librairie.Error);
    LENGTH_LIST_FILE    : StrPCopy(Data, IntToStr(Length(Librairie.FileList)));
    LENGTH_LIST_WARNING : StrPCopy(Data, IntToStr(Length(Librairie.Warning)));
    LENGTH_LIST_ERROR   : StrPCopy(Data, IntToStr(Length(Librairie.Error)));
  end;
end;

function  DeleteFile(const FileName: string; toTrash: boolean; deleteIfRH: boolean): integer;
var Attr   : Integer;
    Confirm: boolean;
    Rec    : TShellFileOpStruct;
begin
  result := 0;
  Attr   := FileGetAttr(FileName);
  if not (Attr and faSysFile > 0) then
  begin
    Confirm := true;
    if (Attr and faHidden > 0) or (Attr and faReadOnly > 0) or (Attr and faSysFile > 0) then Confirm := deleteIfRH;
    if Confirm then
    begin
      result := SizeOfFile(FileName);
      if (Attr and faHidden > 0)   then Attr := Attr - faHidden;
      if (Attr and faReadOnly > 0) then Attr := Attr - faReadOnly;
      FileSetAttr(FileName, Attr);
      if not toTrash then
        SysUtils.DeleteFile(FileName)
      else begin
        Rec.wFunc  := foDelete;
        Rec.pFrom  := FileName;
        Rec.fFlags := [fofAllowUndo, fofFilesOnly, fofNoConfirmation, fofSilent];
        ShellFileOperation(Rec);
      end;
      if FileExists(fileName) then result := 0;
    end;
  end;
end;

function GetCurrentPercent: Real;
begin
  result := Librairie.Pourcent;
end;

function GetCurrentStatus : TCanStats;
begin
  result := Librairie.CurrStat;
end;

(* Librairie - Lock *)
                                     
function LockLib(ID: PChar): boolean;
begin
  result := false;
  if Librairie.LockID <> EmptyStr then Exit;
  result := true;
  Librairie.LockID := StrPas(ID);
end;

function UnLockLib(ID: PChar): boolean;
begin
  result := false;
  if (not (Librairie.LockID <> EmptyStr)) or
     (StrPas(ID) <> Librairie.LockID) then Exit;
  result := true;
  Librairie.LockID := EmptyStr;
end;

function IsLocked: boolean;
begin
  result := Librairie.LockID <> EmptyStr;
end;

procedure NameLocked(ID: PChar);
begin
  if not (Librairie.LockID <> EmptyStr) then Exit;
  StrPCopy(ID, Librairie.LockID);
end;

(* Librairie - Log *)

resourcestring
  Entete = '(**************************************************************)'#10 + // 60
           '(* %s *)'#10 +
           '(*                    Rapport de recherche                    *)'#10 +
           '(* %s *)'#10 +
           '(**************************************************************)'#10 +
           #10 +
           '%s';
const
  LengthTete = 58;

procedure CreateFileLog(FileName, ID, Data: PChar; Erase: boolean);
var ApplicationName, Date, WData: string;
    ListApp                     : TStrings;
    I, Max, Len                 : Integer;

begin
  Date := 'Le ' + DateTimeToStr(Now);
  ListApp := TStringList.Create;
  try
    ListApp.CommaText := Librairie.AppNames;
    try
      I := 0; while not Pos(StrPas(ID), ListApp[I]) = 0 do Inc(I);
      ApplicationName := Copy(ListApp[I], pos('|', ListApp[I]) + 1, length(ListApp[I]) - pos('|', ListApp[I]));
    except
      ApplicationName := 'Unknow Application';
    end;
  finally
    ListApp.Free;
  end;
  Len := (LengthTete - length(Date));
  Max := Len div 2;               for I := 1 to Max do Date := ' ' + Date;
  Max := Len div 2 + (Len Mod 2); for I := 1 to Max do Date := Date + ' ';

  Len := (LengthTete - length(ApplicationName));
  Max := Len div 2;               for I := 1 to Max do ApplicationName := ' ' + ApplicationName;
  Max := Len div 2 + (Len Mod 2); for I := 1 to Max do ApplicationName := ApplicationName + ' ';

  WData := Format(EnTete, [ApplicationName,
                           Date,
                           StrPas(Data)
                          ]);
  ListApp := TStringList.Create;
  try
    try
      If not Erase then ListApp.LoadFromFile(FileName);
    except
    end;
    ListApp.Add(WData);
    ListApp.SaveToFile(FileName);
  finally
    ListApp.Free;
  end;
end;

initialization
  Librairie := nil;

end.
