unit GlobClass;

interface

uses Windows, classes, ShareMemUtils, uSearch, Algo_Rech;

type

{ Enregistrement contenant une partie es informations  partager }

  TBlocMem = record
               LoadedNum,
               AppLinkSz,
               AppLinkAcquire,
               ReturnSize,
               ReturnAcquire : Integer;
               CurrentStats  : TCanStats;
               Pourcent      : real;
               Abort         : boolean;
               LockID        : TXDLID;
             end;
  PBlocMem = ^TBlocMem;

{ TXDLLib }

{ Represente les fonctions principale de la Librairie }

  TXDLLib = class(TThread)
  private
    FStart      : boolean;
    FSharedMem,
    FAppLinkMem,
    FReturnMem  : TSharedMem;
    FMutex      : TMutex;
    FData       : PBlocMem;
    FEndSrchProc: TEndSearchProc;
    FWorkstation: TRAPosteDeTravail;

    function getLoadedNum: Integer;
    function getAppLnkAcq: Integer;
    function getCurrStats: TCanStats;
    function getPercent  : Real;
    function getLockID   : TXDLID;
    function getAbort    : boolean;
    function getReturnAcq: integer;
    function getAList(Index: Integer) : string;
    procedure setReturnAcq(Value: integer);
    procedure setAbort(Value: boolean);
    procedure setLockID(Value: TXDLID);
    procedure setLoadedNum(Value: Integer);
    procedure setAppLnkAcq(Value: Integer);
    procedure setCurrStats(Value: TCanStats);
    procedure setPercent(Value: Real);
    function getAppNames: String;
  protected
    procedure Execute; override;
    function ChangePercent(R: Real): boolean;
  public
    constructor Create(const APPID: TXDLID; const APPNAME, Key: string);
    destructor Destroy; override;

    procedure AddAppName(AppID, AppName: string);
    procedure CleanWindowsTrash;
    function  CleanWindowsTempDirectory: Integer;
    function  InitializeTree(Data: Pointer; Size: Integer): boolean;
    procedure SearchFile(Priority: TThreadPriority);

    procedure setReturnFile(const FileList, Warning, Error: string);

    property AppNames : String         read getAppNames;
    property LoadedNum: Integer        read getLoadedNum write setLoadedNum;
    property AppLnkAcq: Integer        read getAppLnkAcq write setAppLnkAcq;
    property ReturnAcq: Integer        read getReturnAcq write setReturnAcq;
    property CurrStat : TCanStats      read getCurrStats write setCurrStats;
    property Pourcent : Real           read getPercent   write setPercent;
    property LockID   : TXDLID         read getLockID    write setLockID;
    property EndProc  : TEndSearchProc read FEndSrchProc write FEndSrchProc;
    property Abort    : boolean        read getAbort     write setAbort;
    property FileList : string index 1 read getAList;
    property Warning  : string index 2 read getAList;
    property Error    : string index 3 read getAList;
  end;

implementation

uses SysUtils, WinUtils, uSHFileOp, uDiskUtils, uWindows, Thread_Algo;

resourcestring
  XDLSHAREMEM = 'XDLSHARE';  // Mmoire principale partag
  XDLMUTEX    = 'XDLMUTEX';  // Mutex pour grer la mmoire
  XDLAPPMEM   = 'XDLAPPMM';  // Nom des applications qui se connecte
  XDLRETURN   = 'XDLRETURN'; // Fichiers trouv, Warning, Erreur
const
  XDLTimeOut  = 3000;

constructor TXDLLib.Create(const APPID: TXDLID; const APPNAME, Key: string);
begin
  FStart        := false;
  // Cration d'un bloc de mmoire partag ...
  FSharedMem    := TSharedMem.Create(XDLSHAREMEM, sizeOf(TBlocMem));
  FData         := FSharedMem.Buffer;
  // ... et de son mutex.
  FMutex        := TMutex.Create(XDLMUTEX);
  // La librairie est lu une fois de plus
  LoadedNum     := LoadedNum + 1;

  FWorkstation  := nil;

  Priority      := tpIdle;
  inherited Create(false);
                                             
  // On ajoute le om de l'application au programme dmarrer.
  AddAppName(AppID, AppName);
end;

destructor TXDLLib.Destroy;
begin
  inherited Destroy;
  if Assigned(FWorkstation) then FWorkstation.Free;
  // On Ferme une session et on libre la mmoire
  LoadedNum  := LoadedNum - 1;
  FMutex.Free;
  FSharedMem.Free;
end;

{ Nombre de chargement }

procedure TXDLLib.setLoadedNum(Value: Integer);
begin
  FMutex.Get(XDLTimeOut);
  try
    FData.LoadedNum := Value;
  finally
    FMutex.Release;
  end;
end;

function TXDLLib.getLoadedNum: Integer;
begin
  result := FData.LoadedNum;
end;

{ Gestion du nombre de threads qui se connecte }

procedure TXDLLib.setAppLnkAcq(Value: Integer);
begin
  FMutex.Get(XDLTimeOut);
  try
    FData.AppLinkAcquire := Value;
  finally
    FMutex.Release;
  end;
end;

function TXDLLib.getAppLnkAcq: Integer;
begin
  result := FData.AppLinkAcquire;
end;

{ Gestion des ordres de travail }

procedure TXDLLib.setCurrStats(Value: TCanStats);
begin
  FMutex.Get(XDLTimeOut);
  try
    FData.CurrentStats := Value;
  finally
    FMutex.Release;
  end;
end;

function TXDLLib.getCurrStats: TCanStats;
begin
  result := FData.CurrentStats;
end;

function TXDLLib.getPercent  : Real;
begin
  result := FData.Pourcent;
end;

procedure TXDLLib.setPercent(Value: Real);
begin
  FMutex.Get(XDLTimeOut);
  try
    FData.Pourcent := Value;
  finally
    FMutex.Release;
  end;
end;

function TXDLLib.getLockID: TXDLID;
begin
  result := FData.LockID;
end;

procedure TXDLLib.setLockID(Value: TXDLID);
begin
  FMutex.Get(XDLTimeOut);
  try
    FData.LockID := Value;
  finally
    FMutex.Release;
  end;
end;

function TXDLLib.getAbort: boolean;
begin
  result := FData.Abort;
end;

procedure TXDLLib.setAbort(Value: boolean);
begin
  FMutex.Get(XDLTimeOut);
  try
    FData.Abort := Value;
  finally
    FMutex.Release;
  end;
end;

function TXDLLib.getReturnAcq: integer;
begin
  result := FData.ReturnAcquire;
end;

procedure TXDLLib.setReturnAcq(Value: integer);
begin
  FMutex.Get(XDLTimeOut);
  try
    FData.ReturnAcquire := Value;
  finally
    FMutex.Release;
  end;
end;

function TXDLLib.getAList(Index: Integer): string;
var Taille1, Taille2, Taille3: Integer;
    Texte1, Texte2, Texte3   : String;
    P                        : Pointer;
begin
  result := '';
  if not Assigned(FReturnMem) then Exit;
  P := FReturnMem.Buffer;
  Taille1 := Integer(P^); Integer(P) := Integer(P) + SizeOf(Integer);
  Taille2 := Integer(P^); Integer(P) := Integer(P) + SizeOf(Integer);
  Taille3 := Integer(P^); Integer(P) := Integer(P) + SizeOf(Integer);
  setString(Texte1, PChar(P), Taille1); Integer(P) := Integer(P) + Taille1;
  setString(Texte2, PChar(P), Taille2); Integer(P) := Integer(P) + Taille2;
  setString(Texte3, PChar(P), Taille3); Integer(P) := Integer(P) + Taille3;
  case Index of
    1 : result := Texte1;
    2 : result := Texte2;
    3 : result := Texte3;
  end;
end;

procedure TXDLLib.setReturnFile(const FileList, Warning, Error: string);
var Sh: TSharedMem;
    P : Pointer;
begin
  while not FStart do Sleep(10);
  // On demande l'arrt  tous le monde
  FMutex.Get(XDLTimeOut);
  try
    FData.ReturnSize := 0;
  finally
    FMutex.Release;
  end;
  // Et on attent
  while FData.ReturnAcquire > 0 do Sleep(100);

  Sh := TSharedMem.Create(XDLRETURN, 3 * SizeOf(Integer) +
                                     length(FileList)    +
                                     length(Warning)     +
                                     length(Error));
  try
    P           := Sh.Buffer;
    integer(P^) := length(FileList);
      Integer(P) := Integer(P) + SizeOf(Integer);
    integer(P^) := length(Warning);
      Integer(P) := Integer(P) + SizeOf(Integer);
    integer(P^) := length(Error);
      Integer(P) := Integer(P) + SizeOf(Integer);
    Move(PChar(FileList)^, P^, length(FileList));
      Integer(P) := Integer(P) + length(FileList);
    Move(PChar(Warning)^, P^, length(Warning));
      Integer(P) := Integer(P) + length(Warning);
    Move(PChar(Error)^, P^, length(Error));

    // On donne  nouveaux l'accs
    FMutex.Get(XDLTimeOut);
    try
      FData.ReturnSize := Sh.Size;
    finally
      FMutex.Release;
    end;
    // On attend que quelqu'un revienne
    while FData.ReturnAcquire = 0 do Sleep(100);
  finally
    Sh.Free;
  end;
end;

{ Gestions du nom des applications qui se connecte }

function TXDLLib.getAppNames: String;
begin
  result := '';
  if Assigned(FAppLinkMem) then
    result := StrPas(FAppLinkMem.Buffer);
end;

procedure TXDLLib.AddAppName(AppID, AppName: string);
var Sh    : TSharedMem;
    Chaine: TStringList;
    S     : string;
begin
  while not FStart do Sleep(10);
  // On rcupre l'ancienne chaine
  Chaine := TStringList.Create;
  try
    if Assigned(FAppLinkMem) then
      Chaine.CommaText := StrPas(FAppLinkMem.Buffer);
    S := Format('%s|%s', [AppID, AppName]);
    // On ajoute et on retient
    if Chaine.IndexOf(S) < 0 then Chaine.Add(S);
    S := Chaine.CommaText;
  finally
    Chaine.Free;
  end;
  // On demande l'arrt  tous le monde
  FMutex.Get(XDLTimeOut);
  try
    FData.AppLinkSz := 0;
  finally
    FMutex.Release;
  end;
  // Et on attent
  while FData.AppLinkAcquire > 0 do Sleep(100);

  Sh := TSharedMem.Create(XDLAPPMEM, length(S) + 1);
  try
    // on copy la chaine
    StrPCopy(Sh.Buffer, S);

    // On donne  nouveaux l'accs
    FMutex.Get(XDLTimeOut);
    try
      FData.AppLinkSz := length(S) + 1;
    finally
      FMutex.Release;
    end;
    // On attend que quelqu'un revienne
    while FData.AppLinkAcquire = 0 do Sleep(100);
  finally
    Sh.Free;
  end;
end;

{ Gestion de la recherche }

function TXDLLib.InitializeTree(Data: Pointer; Size: Integer): boolean;
var FStreamTree : TMemoryStream;
begin
  // On verify qu'aucune recherche n'est en cours
  if CleanPerso in CurrStat then
  begin
    result := false;
    exit;
  end;

  FStreamTree   := TMemoryStream.Create;
  try
    FStreamTree.setSize(Size);
    if Assigned(Data) then
      Move(Data^, FStreamTree.Memory^, Size);
    FStreamTree.Seek(0, soFromBeginning);
    if Assigned(FWorkstation) then FWorkstation.Free;
    FWorkstation := TRAPosteDeTravail.Create(26);
    FWorkstation.LoadFromStream(FStreamTree);
  finally
    FStreamTree.Free;
  end;
  
  result := true;
end;

procedure TXDLLib.CleanWindowsTrash;
begin
  if CleanTrash in CurrStat then Exit;
  CurrStat := CurrStat + [CleanTrash];
  Pourcent := 0;
  try
    ShellEmptyRecycleBin;
  finally
    CurrStat := CurrStat - [CleanTrash];
    Pourcent := 100;
  end;
  if Assigned(FEndSrchProc) then FEndSrchProc(CleanTrash);
end;

function TXDLLib.CleanWindowsTempDirectory: Integer;
var
  ShellStruct: TShellFileOpStruct;
  TmpDir     : string;
begin
  result := 0;
  if CleanTrash in CurrStat then Exit;
  CurrStat := CurrStat + [CleanTempDir];
  Pourcent := 0;
  try
    TmpDir             := UpperCase (NormalDir(GetTempDir));
    result             := SizeOfDir(TmpDir);
    ShellStruct.wFunc  := foDelete;
    ShellStruct.pFrom  := TmpDir + '*.*';
    ShellStruct.fFlags := [fofNoConfirmation, fofSilent];

    ShellFileOperation(ShellStruct);
    result := result - SizeOfDir(TmpDir);
  finally
    CurrStat := CurrStat - [CleanTempDir];
    Pourcent := 100;
  end;
  if result < 0 then result := 0;
  if Assigned(FEndSrchProc) then FEndSrchProc(CleanTempDir);
end;

procedure TXDLLib.SearchFile(Priority: TThreadPriority);
var Thread                          : TSearchThread;
    ListFile, ListWarning, ListError: TStringList;
begin
  if (CleanTrash in CurrStat) or (not Assigned(FWorkstation)) then Exit;
  CurrStat := CurrStat + [CleanPerso];
  Abort         := false;

  try
    ListFile    := TStringList.Create;
    ListWarning := TStringList.Create;
    ListError   := TStringList.Create;
  except
    ListFile    := nil;
    ListWarning := nil;
    ListError   := nil;
  end;

  try
    Thread             := TSearchThread.Create(FWorkstation, priority, ListFile);
    Thread.WarningList := ListWarning;
    Thread.ErrorList   := ListError;
    Thread.PercentProc := ChangePercent;
    if not Abort then
    begin
      Thread.Resume;
      Thread.WaitFor;
    end;
    setReturnFile(ListFile.CommaText, ListWarning.CommaText, ListError.CommaText);
  finally
    ListFile.Free;
    ListWarning.Free;
    ListError.Free;
    CurrStat := CurrStat - [CleanPerso];
    if Assigned(FEndSrchProc) then FEndSrchProc(CleanPerso);
  end;
end;

function TXDLLib.ChangePercent(R: Real): boolean;
begin
  result   := not Abort;
  Pourcent := R;
end;

{ Execution du thread de la DLL }

procedure TXDLLib.Execute;
begin
  while not Terminated do
  begin
    // Gestion du noms des applications qui se connecte
    if (not Assigned(FAppLinkMem)) and (FData.AppLinkSz > 0) then
    begin
      AppLnkAcq   := AppLnkAcq + 1;
      FAppLinkMem := TSharedMem.Create(XDLAPPMEM, FData.AppLinkSz);
    end else if Assigned(FAppLinkMem) and (FData.AppLinkSz <= 0) then
             begin
               FAppLinkMem.Free;
               FAppLinkMem := nil;
               AppLnkAcq   := AppLnkAcq - 1;
             end;

    // Gestion du resultat
    if (not Assigned(FReturnMem)) and (FData.ReturnSize > 0) then
    begin
      ReturnAcq   := ReturnAcq + 1;
      FReturnMem := TSharedMem.Create(XDLRETURN, FData.ReturnSize);
    end else if Assigned(FReturnMem) and (FData.ReturnSize <= 0) then
             begin
               FReturnMem.Free;
               FReturnMem := nil;
               ReturnAcq  := ReturnAcq - 1;
             end;

    FStart := true;
    Sleep(500);
  end;
  if Assigned(FAppLinkMem) then
  begin
    FAppLinkMem.Free;
    FAppLinkMem := nil;
    AppLnkAcq   := AppLnkAcq - 1;
  end;
end;

end.
