unit Thread_Algo;

interface

uses
  SysUtils, Classes, uSearch, Algo_Rech;

type

{ TSearchThread }

{ Encapsulation de la recherche dans un thread }

  TSearchThread = class(TThread)
  private
    { Dclarations prives }
    FPercentProc   : TRealPercentProc;
    FPosteDeTravail: TRAPosteDeTravail;

    FRapport       : Real;
    FResult        : boolean;
    FWarningList,
    FErrorList,
    FFileList      : TStrings;
    procedure PercentSync;
  protected
    procedure Execute; override;
    function SendPercent(Pourcentage: Real): boolean; virtual;
    procedure setPercentProc(Value: TRealPercentProc); virtual;
    function getString: String; virtual;
  public
    constructor Create(p: TRAPosteDeTravail; Priority: TThreadPriority; FileList:TStrings);
    property PercentProc: TRealPercentProc read FPercentProc write setPercentProc;
    property WarningList: TStrings read FWarningList write FWarningList;
    property ErrorList: TStrings read FErrorList write FErrorList;
    property toString: String read getString;
  end;

implementation

{ TSearchThread }

constructor TSearchThread.Create(p: TRAPosteDeTravail; Priority: TThreadPriority; FileList:TStrings);
begin
  FFileList       := FileList;
  FRapport        := 0;
  FPosteDeTravail := p;

  Self.Priority   := Priority;
  FreeOnTerminate := true;
  inherited Create(true);
end;

procedure TSearchThread.PercentSync;
begin
  FResult := true;
  if Assigned(FPercentProc) then
    FResult := PercentProc(FRapport) or Terminated;
end;

function TSearchThread.SendPercent(Pourcentage: Real): boolean;
begin
  FRapport := Pourcentage;
  Synchronize(PercentSync);
  result := FResult;
end;

procedure TSearchThread.setPercentProc(Value: TRealPercentProc);
begin
  if @Value <> @FPercentProc then
  begin
    FPercentProc := Value;
    if Assigned(FPercentProc) then
      IRechArborescense(FPosteDeTravail).PercentProc := SendPercent
    else
      IRechArborescense(FPosteDeTravail).PercentProc := nil;
  end;
end;

function TSearchThread.getString: String;
begin
  result := FPosteDeTravail.toString;
end;

procedure TSearchThread.Execute;
begin
  try
    FFileList.AddStrings(IRechArborescense(FPosteDeTravail).Execute);
  except
    on e: Exception do
      if Assigned(FErrorList) then
        FErrorList.Add(Format(cErrorMsg, [cThreadError, e.Message]))
  end;
end;

end.

