unit MaskDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, 
  Buttons, BrowseDr;

type
  TMasqueDlg = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    HelpBtn: TButton;
    MskSrcList: TListBox;
    MskDstList: TListBox;
    SrcLabel: TLabel;
    DstLabel: TLabel;
    IncludeBtn: TSpeedButton;
    IncAllBtn: TSpeedButton;
    ExcludeBtn: TSpeedButton;
    ExAllBtn: TSpeedButton;
    Label1: TLabel;
    StaticTextDirName: TStaticText;
    ButtonBrowns: TButton;
    Label2: TLabel;
    EditMasque: TEdit;
    CheckBoxRecurseDst: TCheckBox;
    BrowseDirectoryDlg: TBrowseDirectoryDlg;
    CheckBoxRecurseSrc: TCheckBox;
    procedure ButtonBrownsClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure MskSrcListClick(Sender: TObject);
    procedure MskDstListClick(Sender: TObject);
    procedure MskSrcListKeyPress(Sender: TObject; var Key: Char);
    procedure MskDstListKeyPress(Sender: TObject; var Key: Char);
    procedure CheckBoxRecurseDstClick(Sender: TObject);
    procedure IncAllBtnClick(Sender: TObject);
    procedure ExAllBtnClick(Sender: TObject);
    procedure IncludeBtnClick(Sender: TObject);
    procedure ExcludeBtnClick(Sender: TObject);
    procedure EditMasqueKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    FDirectoryName: string;
    FSrcList,
    FDstList      : TList;
    procedure setDirectoryName(Value: string);
  public
    { Public declarations }
    procedure Refresh;
    procedure SelecteChange;

    property SourceList     : TList  read FSrcList;
    property DestinationList: TList  read FDstList;
    property DirectoryName  : string read FDirectoryName write setDirectoryName;
  end;

function OpenMasqueDlg(var DirName:string; SrcList, DestList: TList; NewDirectory: boolean): boolean;

implementation

{$R *.DFM}

uses uDiskUtils, OptDlg, Algo;

function OpenMasqueDlg(var DirName:string; SrcList, DestList: TList; NewDirectory: boolean): boolean;
var MasqueDlg: TMasqueDlg;
    I, J     : Integer;
begin
  MasqueDlg := TMasqueDlg.CreateParented(OptionsForm.ParentWindow);
  with MasqueDlg do
  try
    ButtonBrowns.Enabled := NewDirectory;
    DirectoryName        := DirName;
    SourceList.Clear;   DestinationList.Clear;
    for I := 0 to SrcList .Count - 1 do SourceList.Add(TMasque(SrcList[I]).copie);
    for I := 0 to DestList.Count - 1 do DestinationList.Add(TMasque(DestList[I]).copie);

    for I := 0 to DestList.Count - 1 do
      for J := 0 to SrcList.Count - 1 do
        if TMasque(SrcList[J]).masque = TMasque(DestList[I]).masque then
          SourceList[J] := nil;
    SourceList.Pack;
    Refresh;

    result  := ShowModal = mrok;
    if result then
    begin
      DirName := directoryName;
      DestList.Clear;
      for I := 0 to DestinationList.Count - 1
        do DestList.Add(DestinationList[I]);
    end;
  finally
    Free;
  end;
end;

procedure TMasqueDlg.setDirectoryName(Value: string);
begin
  if Value <> FDirectoryName then
  begin
    StaticTextDirName.Caption := MinimizeFileName(Value,
            StaticTextDirName.Width div (Canvas.TextWidth(Value) div length(Value)));
    FDirectoryName            := Value;
    SelecteChange;
  end;
end;

procedure TMasqueDlg.ButtonBrownsClick(Sender: TObject);
begin
  if BrowseDirectoryDlg.Execute then
    DirectoryName := BrowseDirectoryDlg.Selection;
end;

procedure TMasqueDlg.FormCreate(Sender: TObject);
begin
  FSrcList := TList.Create;
  FDstList := TList.Create;
end;

procedure TMasqueDlg.FormDestroy(Sender: TObject);
var I: Integer;
begin
  for I := 0 to FSrcList.Count - 1 do TMasque(FSrcList[I]).Free;
  FSrcList.Free;
  FDstlist.Free;
end;

procedure TMasqueDlg.Refresh;
var I: Integer;
begin
  for I := 0 to FSrcList.Count - 1 do MskSrcList.Items.Add(TMasque(FSrcList[I]).masque);
  for I := 0 to FDstList.Count - 1 do MskDstList.Items.Add(TMasque(FDstList[I]).masque);
  SelecteChange;
end;

procedure TMasqueDlg.SelecteChange;
begin
  if (MskSrcList.Items.Count > 0) and (MskSrcList.ItemIndex > -1) then
    CheckBoxRecurseSrc.Checked := TMasque(FSrcList[MskSrcList.ItemIndex]).recurse;
  if (MskDstList.Items.Count > 0) and (MskDstList.ItemIndex > -1) then
    CheckBoxRecurseDst.Checked := TMasque(FDstList[MskDstList.ItemIndex]).recurse;
  CheckBoxRecurseDst.Enabled   := MskDstList.ItemIndex > -1;

  IncludeBtn.Enabled           := MskSrcList.Items.Count > 0;
  IncAllBtn.Enabled            := MskSrcList.Items.Count > 0;
  ExcludeBtn.Enabled           := MskDstList.Items.Count > 0;
  ExAllBtn.Enabled             := MskDstList.Items.Count > 0;
  OKBtn.Enabled                := length(DirectoryName) > 0;                     
end;

procedure TMasqueDlg.MskSrcListClick(Sender: TObject);
begin
  SelecteChange;
end;

procedure TMasqueDlg.MskDstListClick(Sender: TObject);
begin
  SelecteChange;
end;

procedure TMasqueDlg.MskSrcListKeyPress(Sender: TObject; var Key: Char);
begin
  SelecteChange;
end;

procedure TMasqueDlg.MskDstListKeyPress(Sender: TObject; var Key: Char);
begin
  SelecteChange;
end;

procedure TMasqueDlg.CheckBoxRecurseDstClick(Sender: TObject);
begin
  if CheckBoxRecurseDst.Enabled then
    TMasque(FDstList[MskDstList.ItemIndex]).recurse := CheckBoxRecurseDst.Checked;
end;

procedure TMasqueDlg.IncAllBtnClick(Sender: TObject);
var I: Integer;
begin
  for I := FSrcList.Count - 1 downto 0 do
  begin
    MskDstList.Items.Add(MskSrcList.Items[I]);
    MskSrcList.Items.Delete(I);
    FDstList.Add(FSrcList[I]);
    FSrcList.Delete(I);
  end;
  SelecteChange;
end;

procedure TMasqueDlg.ExAllBtnClick(Sender: TObject);
var I: Integer;
begin
  for I := FDstList.Count - 1 downto 0 do
  begin
    MskSrcList.Items.Add(MskDstList.Items[I]);
    MskDstList.Items.Delete(I);
    FSrcList.Add(FDstList[I]);
    FDstList.Delete(I);
  end;
  SelecteChange;
end;

procedure TMasqueDlg.IncludeBtnClick(Sender: TObject);
var I: Integer;
begin
  for I := MskSrcList.Items.Count - 1 downto 0 do
    if MskSrcList.Selected[I] then
    begin
      MskDstList.Items.Add(MskSrcList.Items[I]);
      MskSrcList.Items.Delete(I);
      FDstList.Add(FSrcList[I]);
      FSrcList.Delete(I);
      SelecteChange;
    end;
end;

procedure TMasqueDlg.ExcludeBtnClick(Sender: TObject);
var I: Integer;
begin
  for I := MskDstList.Items.Count - 1 downto 0 do
    if MskDstList.Selected[I] then
    begin
      MskSrcList.Items.Add(MskDstList.Items[I]);
      MskDstList.Items.Delete(I);
      FSrcList.Add(FDstList[I]);
      FDstList.Delete(I);
      SelecteChange;
    end;
end;

procedure TMasqueDlg.EditMasqueKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #13 then
  begin
    MskDstList.Items.Add(EditMasque.Text);
    FDstList.Add(TMasque.Create(EditMasque.Text, true));
    Key := #0;
    EditMasque.Text := '';
  end;
end;

end.
