unit XDLTreeView;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, Imglist, Algo;

type
  TXDLTreeView = class(TCustomTreeView)
  private
    { Dclarations prives }
    FImageList : TImageList;
    FTreeView  : TPosteDeTravail;
    FOnChange  : TNotifyEvent;
    FTreeHide  : TTreeNodes;
    FMasqueList: TList;
  protected
    { Dclarations protges }
    procedure VerifyDisk;
    procedure Loaded; override;
    function SearchName(Value : TTreeNode) : string;
    function SearchTree(Value : string)    : IArborescense;
    function getSelected: IArborescense;
    function getMasqueSel: TMasque;

    procedure NodeChange(Sender: TObject; Node: TTreeNode);

    property TopItem;
    property DropTarget;
  public
    { Dclarations publiques }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    function Add(S: string): IArborescense; overload;
    function Add(Node: IArborescense; S: string): integer; overload;
    procedure Add(Node: IMasque; M: TMasque); overload;
    procedure Add(Node: IMasque; M: TList); overload;
    procedure Delete(Node: IArborescense); reintroduce;
    procedure Clear;
    procedure Expand(I: Integer; recurse: boolean); reintroduce;

    procedure Refresh;
    procedure LoadFromStream(Stream: TStream);
    procedure SaveToStream(Stream: TStream);
  published
    { Dclarations publies }
    property Items         : TPosteDeTravail  read FTreeView;
    property Selected      : IArborescense    read getSelected;
    property MasqueSel     : TMasque          read getMasqueSel;
    property MasqueList    : TList            read FMasqueList;

    property Align;
    property Anchors;
    property AutoExpand;
    property BiDiMode;
    property BorderStyle;
    property BorderWidth;
    property ChangeDelay;
    property Color;
    property Ctl3D;
    property Constraints;
    property Enabled;
    property Font;
    property HideSelection;
    property HotTrack;
    property ParentBiDiMode;
    property ParentColor default False;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property RightClickSelect;
    property RowSelect;
    property ShowButtons;
    property ShowHint;
    property ShowLines;
    property TabOrder;
    property TabStop default True;
    property ToolTips;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

procedure Register;

implementation

{$R IMGLIST.RES}

uses uDiskUtils;

resourcestring
  rcWorkstation = 'Poste de travail';

function Retreive(var S:string): string;
begin
  result := Copy(S, 1, pos('\', S) - 1);
  System.Delete(S, 1, pos('\', S));
  if (length(result) = 2) and (result[2] = ':') then System.Delete(result, 2, 1);
end;

constructor TXDLTreeView.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FImageList := TImageList.CreateSize(16, 16);

  FImageList.ResourceLoad (rtBitmap, 'TREEVIEWICON', $ff00ff);
  FTreeView  := TPosteDeTravail.Create(26);
  VerifyDisk;
  FMasqueList:= TList.Create;
  FTreeHide  := inherited Items;
  inherited OnChange := NodeChange;

  Images   := FImageList;
  ShowRoot := true;
  ReadOnly := true;
end;

destructor TXDLTreeView.Destroy;
var I: Integer;
begin
  ChangeArborescenseMasque(FTreeView, FMasqueList);
  for I := 0 to FMasqueList.Count - 1 do TMasque(FMasqueList[I]).Free;
  FMasqueList.Free;

  Images := nil;
  FImageList.Free;
  inherited Destroy;
end;

procedure TXDLTreeView.Loaded;
begin
  inherited Loaded;
  refresh;
end;

procedure TXDLTreeView.VerifyDisk;
var Num: Integer;
begin
  Num := GetNumDrive;
  while Num > FTreeView.Count do
  begin
    FTreeView.Add(Chr(FTreeView.Count + Ord('A')));
  end;
end;

procedure TXDLTreeView.Refresh;
  procedure AssignTree(Node: TTreeNode; Lien: IArborescense);
  var I, Drv: Integer;
      Nd    : TTreeNode;
  begin
    for I:= 0 to Lien.Count - 1 do
    begin
      Nd := FTreeHide.AddChild(Node, Lien[I].name);
      if Lien[I].ClassTree = ctDisque then
      begin
        case DriveType (Lien[I].name[1]) of
          Removable  : Drv := 1;
          Fixed      : Drv := 2;
          CDROM      : Drv := 3;
          Remote     : Drv := 4;
          no_root    : Drv := -2;
          else         Drv := -1;
        end;
        Nd.ImageIndex    := Drv;
        Nd.SelectedIndex := Drv;
      end else
      begin
        Drv              := 5;
        Nd.ImageIndex    := 5;
        Nd.SelectedIndex := 6;
      end;
      if Drv = -2 then Nd.Delete else
      AssignTree(Nd, Lien[I]);
    end;
    if Lien.ClassTree <> ctWorkstation then
    for I := 0 to (Lien as IMasque).Masques.Count - 1 do
    begin
      Nd := FTreeHide.AddChild(Node, (Lien as IMasque).Masques[I].masque);
      Nd.ImageIndex    := 7;
      Nd.SelectedIndex := 7;
    end;
  end;
var Node: TTreeNode;
begin
  FTreeHide.BeginUpdate;
  try
    FTreeHide.Clear;
    Node := FTreeHide.Add(nil, rcWorkstation);
    with Node do
    begin
      ImageIndex    := 0;
      SelectedIndex := 0;
    end;
    AssignTree(Node, FTreeView);
  finally
    FTreeHide.EndUpdate;
  end;
  FTreeHide[0].Expand(false);
end;

procedure TXDLTreeView.LoadFromStream(Stream: TStream);
begin
  FTreeView.LoadFromStream(Stream);
  ChangeArborescenseMasque(FTreeView, FMasqueList);
  VerifyDisk;
  Refresh;
end;

procedure TXDLTreeView.SaveToStream(Stream: TStream);
begin
  FTreeView.SaveToStream(Stream);
end;

function TXDLTreeView.SearchName(Value : TTreeNode) : string;
var Position : TTreeNode;
begin
  result   := '';
  Position := Value;
  While Position.Level > 0 do
  begin
    if Position.Level > 1
      then result := Position.Text +  '\' + Result
      else result := Position.Text + ':\' + Result;
    Position := Position.Parent;
  end;
end;

function TXDLTreeView.SearchTree(Value : string): IArborescense;
var Index: Integer;
begin
  result := FTreeView;
  if Value = '' then Exit;
  repeat
    Index := Result.indexOf(Retreive(Value));
    if Index >=0 then
      result := Result[Index];
  until (Index < 0) or (Value = '');
end;

function TXDLTreeView.getSelected: IArborescense;
begin
  if not Assigned(inherited Selected) then result := nil else
  result := SearchTree(SearchName(inherited Selected));
end;

function TXDLTreeView.getMasqueSel: TMasque;
var Path1, path2: string;
    Select      : IArborescense;
begin
  result := nil;
  Select := Selected;
  if not Assigned(Select) then exit;
  path1  := UpperCase(SearchName(inherited Selected));
  path2  := UpperCase(Select.path);
  if (path1 <> path2) and (Select.ClassTree <> ctWorkstation) then
  begin
    System.Delete(path1, 1, length(path2));
    System.delete(path1, length(path1), 1);
    result := (Select as IMasque).Masques[
                (Select as IMasque).Masques.indexOf(TMasque.Create(path1, false))
                                           ];
  end;
end;

procedure TXDLTreeView.NodeChange(Sender: TObject; Node: TTreeNode);
begin
  if Assigned(FOnChange) then FOnChange(Sender);
end;

function TXDLTreeView.Add(S: string): IArborescense;
var I      : Integer;
    Path   : string;
begin
  result := FTreeView;
  repeat
    path := Retreive(S);
    I    := result.IndexOf(path);
    if I > -1 then
      result := result[I];
  until (I = -1) or (S = '');
  if (S = '') and (I > -1) then  Exit;
  repeat
    result := result[Add(result, Path)];
    path    := Retreive(S);
  until S = '';
  if Path <> '' then result := result[Add(result, Path)];
end;

function TXDLTreeView.Add(Node: IArborescense; S: string): integer;
begin
  result := -1;
  if Node.ClassTree <> ctWorkstation then
  begin
    result := Node.Add(S);
    Refresh;
  end;
end;

procedure TXDLTreeView.Add(Node: IMasque; M: TMasque);
begin
  Node.Masques.Add(M);
  ChangeArborescenseMasque(FTreeView, FMasqueList);
  Refresh;
end;

procedure TXDLTreeView.Add(Node: IMasque; M: TList);
var I: Integer;
begin
  for I := 0 to M.count - 1 do Node.Masques.Add(M[I]);
  ChangeArborescenseMasque(FTreeView, FMasqueList);
  Refresh;
end;

procedure TXDLTreeView.Delete(Node: IArborescense);
begin
  if Node.ClassTree = ctDossier then
  begin
    Node.parent.Delete(Node.Parent.IndexOf(Node.Name));
    Refresh;
  end;
end;

procedure TXDLTreeView.Expand(I: Integer; recurse: boolean);
begin
  FTreeHide[I].Expand(recurse);
end;

procedure TXDLTreeView.Clear;
begin
  FTreeView.Free;
  FTreeView  := TPosteDeTravail.Create(26);
  VerifyDisk;
  Refresh;
end;

procedure Register;
begin
  RegisterComponents('VDH Controls', [TXDLTreeView]);
end;

end.
