unit Algo;

interface

uses
  Windows, Classes, SysUtils;

type

{ Type d'objet }

  TClassTree = (ctWorkstation, ctDisque, ctDossier);

{ TMasque }

{ Represente un masque de fichier tel que *.bak, ... }

  TMasque = class(tObject)
  private
    FMasque : String;
    FRecurse: boolean;
  protected
    function getString : string; virtual;
    function getCopie: TMasque; virtual;
  public
    constructor Create(const Name: String; Recurse: boolean);

    procedure LoadFromStream(S: TStream); virtual;
    procedure SaveToStream(S: TStream); virtual;

    property masque : String read FMasque write FMasque;
    property recurse: boolean read FRecurse write FRecurse;
    property toString: String read getString;
    property copie: TMasque read getCopie;
  end;

{ TMasqueList }

{ Represente une liste de Masque }

  TMasqueList = class(tPersistent)
  private
    FItems: TList;
  protected
    function getItem(I: Integer): TMasque; virtual;
    function getString : string; virtual;
    function getCount : Integer; virtual;

    procedure AssignTo(Dest: TPersistent); override;
  public
    constructor Create(AllocSize: Integer);
    destructor Destroy; override;

    function Add(Masque: TMasque) : Integer;
    procedure Delete(Index: Integer);
    function IndexOf(Masque: TMasque): Integer;
    procedure Clear;

    property toString: String read getString;
    property Items[I: Integer] : TMasque read getItem; default;
    property Count: Integer read getCount;
  end;

{ IArborescense }

{ Interface qui represente une Arborescense }

  TArborescense = class;
  IArborescense = interface
    ['{1A60DABB-DCF2-4F2B-B781-9C995991819D}']
    // Protected
    function getParent: IArborescense;
    procedure setParent(value: IArborescense);

    function getName  : string;
    procedure setName(value: string);

    function getLevel : integer;
    function getPath  : string;

    function getCount: Integer;
    function getItem(I: Integer) : IArborescense;

    function getString : string;
    function getClassTree : TClassTree;

    procedure AssignTo(Dest: TPersistent);

    // Public
    procedure Assign(Source: TPersistent);

    procedure LoadFromStream(S: TStream);
    procedure SaveToStream(S: TStream);

    function Add(Arbre: TArborescense): Integer; overload;
    function Add(Name: String): Integer; overload;
    procedure Delete(I: Integer);
    function IndexOf(Path: string): Integer;

    property toString: String read getString;

    property parent   : IArborescense read getParent;
    property level    : integer       read getLevel;
    property path     : string        read getPath;
    property name     : string        read getName write setName;
    property ClassTree: TClassTree    read getClassTree;

    property Items [I: integer] : IArborescense read getItem; default;
    property Count : Integer read getCount;
  end;

{ IMasque }

{ Represente un ensemble de masque }

  IMasque = interface
    ['{6F9516E4-CF3C-475E-B2E9-F2949E281E8C}']
    // Protected
    function getString : string;
    function getMasqueList: TMasqueList;
    // Public
    property toString: String read getString;
    property Masques: TMasqueList read getMasqueList;
  end;

{ TArborescense }

{ Implemente les fonctions de IUnknown }

  TArborescense = class(TPersistent, IArborescense)
  private
    FItems   : TList;
    FParent  : IArborescense;
    FName    : string;
  protected
    function QueryInterface(const IID: TGUID; out Obj): HResult; stdcall;
    function _AddRef: Integer; stdcall;
    function _Release: Integer; stdcall;

    function getParent: IArborescense; virtual;
    procedure setParent(value: IArborescense); virtual;

    function getName  : string; virtual;
    procedure setName(value: string); virtual;

    function getLevel : integer; virtual;
    function getPath  : string; virtual;

    function getItem(I: Integer) : IArborescense; virtual;
    function getCount: Integer; virtual;

    function getString : string; virtual;
    function getClassTree : TClassTree; virtual; abstract;

    procedure AssignTo(Dest: TPersistent); override;
  public
    constructor Create(Name: string; Parent: IArborescense; AllocSize: Integer);
    destructor Destroy; override;

    procedure LoadFromStream(S: TStream); virtual; abstract;
    procedure SaveToStream(S: TStream); virtual; abstract;

    function Add(Arbre: TArborescense): Integer; overload; virtual;
    function Add(Name: String): Integer; overload; virtual;
    procedure Delete(I: Integer); virtual;
    function IndexOf(Path: string): Integer;

    property toString: String read getString;

    property parent   : IArborescense read getParent;
    property level    : integer       read getLevel;
    property path     : string        read getPath;
    property name     : string        read getName write setName;
    property ClassTree: TClassTree    read getClassTree;

    property Items [I: integer] : IArborescense read getItem; default;
    property Count : Integer read getCount;
  end;

{ TPosteDeTravail }

{ Represente la racine possdent tous les disques }

  TPosteDeTravail = class(TArborescense, IArborescense)
  protected
    function getParent: IArborescense; override;
    procedure setParent(value: IArborescense); override;

    function getName  : string; override;
    procedure setName(value: string); override;

    function getLevel : integer; override;
    function getPath  : string; override;

    function getClassTree : TClassTree; override;
  public
    constructor Create(AllocSize: Integer);
    function Add(Arbre: TArborescense): Integer; override;
    function Add(Name: String): Integer; override;

    procedure LoadFromStream(S: TStream); override;
    procedure SaveToStream(S: TStream); override;
  end;

{ TDossier }

{ Represente un dossier }

  TDossier = class(TArborescense, IArborescense, IMasque)
  private
    FMasques : TMasqueList;
  protected
    function getMasqueList: TMasqueList; virtual;
    function getString : string; override;
    function getClassTree : TClassTree; override;
    procedure AssignTo(Dest: TPersistent); override;
  public
    constructor Create(Name: string; Parent: IArborescense; AllocSize, AllocMask: Integer);
    destructor Destroy; override;

    procedure LoadFromStream(S: TStream); override;
    procedure SaveToStream(S: TStream); override;

    property Masques : TMasqueList read getMasqueList;
  end;

{ TDisque }

{ Represente un disque }

  TDisque = class(TDossier, IArborescense, IMasque)
  protected
    procedure setName(value: string); override;
    function getPath  : string; override;
    function getClassTree : TClassTree; override;
  public
    constructor Create(Name: char; Parent: IArborescense; AllocSize, AllocMask: Integer);
  end;

procedure ChangeArborescenseMasque(Racine: IArborescense; Liste: TList);

implementation

resourcestring
  WORKSTATION = 'Poste de travail';
  ST_RECURSE  = 'Recursif';

{ Utilitaire }

procedure ChangeArborescenseMasque(Racine: IArborescense; Liste: TList);
  function IndexOf(List: TList; Masque: TMasque): Integer;
  begin
    for result := 0 to List.Count - 1 do
      if (TMasque(List[Result]).masque  = Masque.masque)  and
         (TMasque(List[Result]).recurse = Masque.recurse) then
        Exit;
    result := -1;
  end;
var I, Index: Integer;
    root    : IMasque;
begin
  try
    root := Racine as IMasque;
    for I := 0 to root.Masques.Count - 1 do
    begin
      Index := IndexOf(Liste, root.Masques[I]);
      if Index <> -1 then
      begin
        if not (Liste[Index] = root.Masques[I]) then
        begin
          root.Masques[I].Free;
          root.Masques.Delete(I);
          root.Masques.Add(Liste[Index]);
        end
      end else
        Liste.Add(root.Masques[I]);
    end;
  except
  end;
  for I := 0 to Racine.Count - 1 do
    ChangeArborescenseMasque(Racine.Items[I], Liste);
end;

procedure WriteStringInto(S: TStream; St: String);
var _St: ShortString;
begin
  FillChar(_St, SizeOf(ShortString), #0);
  _St := St;
  S.Write(_St, length(_St) + 1);
end;

function ReadStringFrom(S: TStream) : String;
var I: Byte;
  _St: ShortString;
begin
  S.Read(I, 1);
  S.Seek(-1, soFromCurrent);
  S.Read(_St, I + 1);
  result := _St;
end;

{ TMasque }

constructor TMasque.Create(const Name: String; Recurse: boolean);
begin
  FMasque := name;
  FRecurse:= recurse;
end;

function TMasque.getString : string;
begin
  result := FMasque;
  if FRecurse then result := result + ' - [' + ST_RECURSE + ']';
end;

procedure TMasque.LoadFromStream(S: TStream);
begin
  FMasque := ReadStringFrom(S);
  S.Read(FRecurse, SizeOf(boolean));
end;

procedure TMasque.SaveToStream(S: TStream);
begin
  WriteStringInto(S, FMasque);
  S.Write(FRecurse, SizeOf(boolean));
end;

function TMasque.getCopie: TMasque;
begin
  result := TMasque.Create(FMasque, FRecurse);
end;

{ TMasqueList }

constructor TMasqueList.Create(AllocSize: Integer);
begin
  FItems := TList.Create;
  FItems.Capacity := AllocSize;
end;

procedure TMasqueList.AssignTo(Dest: TPersistent);
var I: Integer;
begin
  For I := 0 to Count - 1 do
    TMasqueList(Dest).Add(Items[I]);
end;

procedure TMasqueList.Clear;
begin
  FItems.Clear;
end;

destructor TMasqueList.Destroy;
begin
  FItems.Free;
end;

function TMasqueList.getItem(I: Integer): TMasque;
begin
  result := TMasque(FItems[I]);
end;

function TMasqueList.getString : string;
var I: Integer;
begin
  result := '';
  for I := 0 to Count - 1 do
    result := result + Items[I].toString + #10;
end;

function TMasqueList.getCount : Integer;
begin
  result := FItems.Count;
end;

function TMasqueList.Add(Masque: TMasque) : Integer;
begin
  result := FItems.Add(Masque);
end;

procedure TMasqueList.Delete(Index: Integer);
begin
  FItems.Delete(Index);
end;

function TMasqueList.IndexOf(Masque: TMasque): Integer;
begin
  for result := 0 to Count - 1 do
    if UpperCase(Items[result].masque) = UpperCase(Masque.masque) then
      Exit;
  result := -1;
end;

{ TArborescense }

constructor TArborescense.Create(Name: string; Parent: IArborescense; AllocSize: Integer);
begin
  Self.name := name;
  FItems    := TList.Create;
  FItems.Capacity := AllocSize;
  FParent   := Parent;
end;

destructor TArborescense.Destroy;
var I: Integer;
begin
  For I := 0 to FItems.Count - 1 do
    TArborescense(FItems[I]).Free;
  FItems.Free;
end;

// Oblig pour les interfaces !!!!
function TArborescense.QueryInterface(const IID: TGUID; out Obj): HResult;
const
  E_NOINTERFACE = $80004002;
begin
  if GetInterface(IID, Obj) then Result := 0 else Result := E_NOINTERFACE;
end;

function TArborescense._AddRef: Integer;
begin
  Result := -1;
end;

function TArborescense._Release: Integer;
begin
  Result := -1;
end;

// Pour Assigner
procedure TArborescense.AssignTo(Dest: TPersistent);
var
  Destination: IArborescense;
  I, Index   : Integer;
begin
  Destination := Dest as TArborescense;
  Destination.Name := Self.FName;
  For I := 0 to Self.Count - 1 do
  begin
    Index := Destination.Add('A');
    Self.Items[I].AssignTo((Dest as TArborescense).FItems[Index]);
  end;
end;

function TArborescense.getParent: IArborescense;
begin
  result := FParent;
end;

procedure TArborescense.setParent(value: IArborescense);
begin
  if FParent <> value then
    FParent := value;
end;

function TArborescense.getName  : string;
begin
  result := FName;
end;

procedure TArborescense.setName(value: string);
begin
  if FName <> value then
    FName := Value;
end;

function TArborescense.getLevel : integer;
begin
  if Assigned(FParent) then
    result := FParent.level + 1
  else
    result := 0;
end;

function TArborescense.getPath  : string;
begin
  if Assigned(FParent) then
    result := FParent.path + FName + '\'
  else
    result := FName + '\';
end;

function TArborescense.getItem(I: Integer) : IArborescense;
begin
  result := TArborescense(FItems[I]);
end;

function TArborescense.Add(Arbre: TArborescense): Integer;
begin
  result := FItems.Add(Arbre);
  Arbre.setParent(Self);
end;

function TArborescense.Add(Name: String): Integer;
begin
  result := Add(TDossier.Create(Name, Self, FItems.Capacity, 16));
end;

procedure TArborescense.Delete(I: Integer);
begin
  FItems.Delete(I);
end;

function TArborescense.IndexOf(Path: string): Integer;
begin
  For Result := 0 to Count - 1 do
    if(UpperCase(Path) = UpperCase(Items[Result].name)) then
      Exit;
  result := -1;
end;

function TArborescense.getCount: Integer;
begin
  result := FItems.Count;
end;

function TArborescense.getString : string;
var
  I: Integer;
  S: string;
begin
  s := '';
  for i := 1 to Level do s := s + #9;
  result := S + Path;
  for I := 0 to Count - 1 do
  result := result + #10 + Items[I].toString;
end;

{ TPosteDeTravail }

constructor TPosteDeTravail.Create(AllocSize: Integer);
begin
  inherited Create(WORKSTATION, nil, AllocSize);
end;

function TPosteDeTravail.Add(Arbre: TArborescense): Integer;
begin
  result := -1;
  if Arbre is TDisque then result := FItems.Add(Arbre);
end;

function TPosteDeTravail.Add(Name: String): Integer;
begin
  result := Add(TDisque.Create(Name[1], Self, FItems.Capacity, 16));
end;

function TPosteDeTravail.getClassTree : TClassTree;
begin
  result := ctWorkStation;
end;

function TPosteDeTravail.getParent: IArborescense;
begin
  result := nil;
end;

procedure TPosteDeTravail.setParent(value: IArborescense);
begin
  // Pas de parent
end;

function TPosteDeTravail.getName  : string;
begin
  result := WORKSTATION;
end;

procedure TPosteDeTravail.setName(value: string);
begin
  // On ne peut pas changer de nom
end;

function TPosteDeTravail.getLevel : integer;
begin
  result := 0;
end;

function TPosteDeTravail.getPath  : string;
begin
  result := '<' + WORKSTATION + '>';
end;

procedure TPosteDeTravail.LoadFromStream(S: TStream);
var I, FCount: Integer;
    Arbre    : IArborescense;
begin
  S.Read(FCount, SizeOf(Integer));
  FItems.Clear;
  for I := 0 to FCount - 1 do
  begin
    Arbre := Items[Add('A')];
    Arbre.LoadFromStream(S);
  end;
end;

procedure TPosteDeTravail.SaveToStream(S: TStream);
var I, FCount: Integer;
begin
  FCount := Count;
  S.Write(FCount, SizeOf(Integer));
  for I := 0 to FCount - 1 do
    Items[I].SaveToStream(S);
end;

{ TDossier }

constructor TDossier.Create(Name: string; Parent: IArborescense; AllocSize, AllocMask: Integer);
begin
  inherited Create(name, parent, AllocSize);
  FMasques := TMasqueList.Create(AllocMask);
end;

destructor TDossier.Destroy;
begin
  FMasques.Free;
  inherited Destroy;
end;

// Pour Assigner

procedure TDossier.AssignTo(Dest: TPersistent);
begin
  inherited AssignTo(Dest);
  Self.Masques.AssignTo((Dest as TDossier).Masques);
end;

function TDossier.getString : string;
var I: Integer;
    S: String;
begin
  result := inherited getString;
  if Masques.Count = 0 then Exit;
  for i := 1 to Level do s := s + #9;
  result := result + #10 + S + Masques.toString;
end;

function TDossier.getClassTree : TClassTree;
begin
  result := ctDossier;
end;

function TDossier.getMasqueList: TMasqueList;
begin
  result := FMasques;
end;

procedure TDossier.LoadFromStream(S: TStream);
var I, FCount: Integer;
    Masque   : TMasque;
    Arbre    : IArborescense;
begin
  FName := ReadStringFrom(S);

  FItems.Clear;
  S.Read(FCount, SizeOf(Integer));
  for I := 0 to FCount - 1 do
  begin
    Arbre := Items[Add(#0)];
    Arbre.LoadFromStream(S);
  end;

  FMasques.Clear;
  S.Read(FCount, SizeOf(Integer));
  for I := 0 to FCount - 1 do
  begin
    Masque := TMasque.Create(#0, false);
    Masque.LoadFromStream(S);
    FMasques.Add(Masque);
  end;
end;

procedure TDossier.SaveToStream(S: TStream);
var I, FCount: Integer;
begin
  WriteStringInto(S, FName);

  FCount := Count;
  S.Write(FCount, SizeOf(Integer));
  for I := 0 to FCount - 1 do
    Items[I].SaveToStream(S);

  FCount := Masques.Count;
  S.Write(FCount, SizeOf(Integer));
  for I := 0 to FCount - 1 do
    Masques[I].SaveToStream(S);
end;

{ TDisque }

constructor TDisque.Create(Name: char; Parent: IArborescense; AllocSize, AllocMask: Integer);
begin
  inherited Create(Name, Parent, AllocSize, AllocMask);
end;

procedure TDisque.setName(value: string);
begin
  if (length(value) > 0) and ('A' <= UpCase(value[1])) and ('Z' >= UpCase(value[1])) then
    inherited setName(UpCase(value[1]));
end;

function TDisque.getPath  : string;
begin
  result := FName + ':\';
end;

function TDisque.getClassTree : TClassTree;
begin
  result := ctDisque;
end;

end.
