unit LibraryClass;

interface

uses Windows, Classes, SysUtils, Controls, uSearch;

type
  EXDLLib     = class(Exception);

  TXDLLib = class(TThread)
  private
    FID         : string;
    FCreated    : boolean;
    FLibrairie  : LongInt;
    FRefresh    : Integer;
    FLocked     : boolean;

    FInitLibrarie      : TInitLibrarie;
    FFreeLibrarie      : TFreeLibrarie;
    FGetNbLibrairy     : TGetNbLibrairy;
    FLengthAppNames    : TLengthAppNames;
    FGetAppNames       : TGetAppNames;
    FgetSearchLists    : TgetSearchLists;
    FsetEndProc        : TsetEndProc;
    FGetCurrentPercent : TGetCurrentPercent;
    FGetCurrentStatus  : TGetCurrentStatus;
    FInitPersoTree     : TInitPersoTree;
    FCleanWindows      : TCleanWindows;
    FDeleteFile        : TDeleteFile;

    FLockLib           : TLockLib;
    FUnLockLib         : TUnLockLib;
    FIsLocked          : TIsLocked;
    FNameLocked        : TNameLocked;

    FCreateFileLog     : TCreateFileLog;

    FPourcentP         : TRealPercentProc;
  protected
    procedure Init(const Name, Key: string);
    procedure Close;
    function LoadingNum : Integer;
    function getAppNames: string;
    function getCurrStat: TCanStats;
    function getLockedID: string;

    procedure Execute; override;
    procedure SendPercent; virtual;
    procedure setEndSearchProc(Value: TEndSearchProc2);
    function getAList(index: Integer): string;
  public
    constructor Create(ID, AppName, Key: string);
    destructor Destroy; override;

    function Lock: boolean;
    function UnLock: boolean;

    procedure InitializeTree(S: TStream);

    procedure CleanTheTrash;
    function  CleanTemp: Integer;
    procedure CleanPerso;
    procedure Abort;
    function DeleteFile(const FileName: string; toTrash: boolean; deleteIfRH: boolean): integer;

    procedure WriteToLogFile(FileName, Data: string; Erase: boolean);

    property NChargement : Integer   read LoadingNum;
    property AppNames    : string    read getAppNames;
    property RefreshTime : Integer   read FRefresh write FRefresh;
    property SearchStat  : TCanStats read GetCurrStat;
    property IsLocked    : boolean   read FLocked;
    property LockedID    : string    read getLockedID;

    property FileList    : string index 1 read getAList;
    property WarningList : string index 2 read getAList;
    property ErrorList   : string index 3 read getAList;

    property PourcentageProc: TRealPercentProc read FPourcentP write FPourcentP;
    property EndProc        : TEndSearchProc2  write setEndSearchProc;
  end;

implementation

var Proc: TEndSearchProc2;

procedure CallProc2(EndStat: TCanStat);
begin
  if Assigned(Proc) then Proc(EndStat);
end;

procedure tXDLLib.Init(const Name, Key: string);
begin
  if not FCreated then Exit;
  FInitLibrarie(PChar(FID), PChar(Name), PChar(Key));
end;

procedure tXDLLib.Close;
begin
  if not FCreated then Exit;
  FFreeLibrarie;
end;

function tXDLLib.LoadingNum : Integer;
begin
  result := 0;
  if not FCreated then Exit;
  result := FGetNbLibrairy;
end;

function tXDLLib.getAppNames: string;
var len: Integer;
    S  : PChar;
begin
  if not FCreated then Exit;
  len := FLengthAppNames;
  GetMem(S, len + 1);
  try
    FGetAppNames(S);
    Result := StrPas(S);
  finally
    FreeMem(S);
  end;
end;

procedure tXDLLib.CleanTheTrash;
begin
  if (not FCreated) or (isLocked and (LockedID <> FID)) then Exit;
  FCleanWindows(CLEAN_TRASH_PROC);
end;

function  tXDLLib.CleanTemp: Integer;
begin
  result := -1;
  if (not FCreated) or (isLocked and (LockedID <> FID)) then Exit;
  result := FCleanWindows(CLEAN_TEMP_PROC);
end;

procedure tXDLLib.CleanPerso;
begin
  if (not FCreated) or (isLocked and (LockedID <> FID)) then Exit;
  FCleanWindows(CLEAN_PERSO_PROC);
end;

function tXDLLib.getAList(index: Integer): string;
var TextLength, TextIndex: Integer;
    PLength, Text        : PChar;
begin
  if not FCreated then Exit;
  TextLength := 0;
  TextIndex  := 0;
  case index of
    1 : begin
          TextLength := LENGTH_LIST_FILE;
          TextIndex  := LIST_FILE;
        end;
    2 : begin
          TextLength := LENGTH_LIST_WARNING;
          TextIndex  := LIST_WARNING;
        end;
    3 : begin
          TextLength := LENGTH_LIST_ERROR;
          TextIndex  := LIST_ERROR;
        end;
  end;

  GetMem(PLength, 255);
  try
    FgetSearchLists(TextLength, PLength);
    TextLength := StrToInt(PLength);
  finally
    FreeMem(PLength);
  end;

  GetMem(Text, TextLength + 1);
  try
    FgetSearchLists(TextIndex, Text);
    result := Text;
  finally
    FreeMem(Text);
  end;
end;

procedure tXDLLib.Abort;
begin
  if not FCreated then Exit;
  FCleanWindows(CLEAN_ABORT_PROC);
end;

procedure tXDLLib.InitializeTree(S: TStream);
var Data: Pointer;
    Size: Integer;
begin
  if not FCreated then Exit;
  Size := S.Size;
  GetMem(Data, size);
  try
    Move((S as TMemoryStream).Memory^, Data^, Size);
    FInitPersoTree(Data, Size);
  finally
    FreeMem(Data);
  end;
end;

function tXDLLib.DeleteFile(const FileName: string; toTrash: boolean; deleteIfRH: boolean): integer;
begin
  result := 0;
  if not FCreated then Exit;
  result := FDeleteFile(FileName, toTrash, deleteIfRH);
end;

procedure tXDLLib.WriteToLogFile(FileName, Data: string; Erase: boolean);
begin
  if (not FCreated) or (isLocked and (LockedID <> FID)) then Exit;
  FCreateFileLog(PChar(FileName), PChar(FID), PChar(Data), Erase);
end;

function tXDLLib.getCurrStat: TCanStats;
begin
  if not FCreated then Exit;
  result := FGetCurrentStatus;
end;

function tXDLLib.getLockedID: string;
var S: Pchar;
begin
  if not FCreated then Exit;
  GetMem(S, SizeOf(TXDLID) + 1);
  try
    FNameLocked(S);
    result := StrPas(S);
  finally
    FreeMem(S);
  end;
end;

function tXDLLib.Lock: boolean;
begin
  result := false;
  if not FCreated then Exit;
  Result := FLockLib(PChar(FID));
end;

function tXDLLib.UnLock: boolean;
begin
  result := false;
  if not FCreated then Exit;
  Result := FUnLockLib(Pchar(FID));
end;

procedure tXDLLib.setEndSearchProc(Value: TEndSearchProc2);
begin
  if not FCreated then Exit;
  Proc := Value;
end;

constructor tXDLLib.Create(ID, AppName, Key: string);
begin
  FLocked      := true;
  Randomize;
  FID          := Format(ID, [Random(99)]);
  FCreated     := false;
  FLibrairie   := LoadLibrary('xdlSearch.dll');
  if FLibrairie >= HINSTANCE_ERROR then
  begin
    @FInitLibrarie      := GetProcAddress(FLibrairie, 'InitLibrarie');
    @FFreeLibrarie      := GetProcAddress(FLibrairie, 'FreeLibrarie');
    @FGetNbLibrairy     := GetProcAddress(FLibrairie, 'GetNbLibrairy');
    @FLengthAppNames    := GetProcAddress(FLibrairie, 'LengthAppNames');
    @FGetAppNames       := GetProcAddress(FLibrairie, 'GetAppNames');
    @FgetSearchLists    := GetProcAddress(FLibrairie, 'getSearchLists');
    @FsetEndProc        := GetProcAddress(FLibrairie, 'setEndProc');
    @FGetCurrentPercent := GetProcAddress(FLibrairie, 'GetCurrentPercent');
    @FGetCurrentStatus  := GetProcAddress(FLibrairie, 'GetCurrentStatus');
    @FInitPersoTree     := GetProcAddress(FLibrairie, 'InitPersoTree');
    @FCleanWindows      := GetProcAddress(FLibrairie, 'CleanWindows');
    @FDeleteFile        := GetProcAddress(FLibrairie, 'DeleteFile');

    @FLockLib           :=  GetProcAddress(FLibrairie, 'LockLib');
    @FUnLockLib         :=  GetProcAddress(FLibrairie, 'UnLockLib');
    @FIsLocked          :=  GetProcAddress(FLibrairie, 'IsLocked');
    @FNameLocked        :=  GetProcAddress(FLibrairie, 'NameLocked');

    @FCreateFileLog     :=  GetProcAddress(FLibrairie, 'CreateFileLog');
    FCreated            := Assigned(FInitLibrarie);
    try
      Init(AppName, Key);
      FsetEndProc(CallProc2);
    except
      FCreated := false;
    end;
    if not FCreated then FreeLibrary(FLibrairie);
  end;
  Priority := tpIdle;
  inherited Create(not FCreated);

  if not FCreated then
    raise EXDLLib.CreateFmt(cErrorMsg, [cNoLibLoad, 'Can''t load xdlSearch.dll']);
end;

destructor tXDLLib.Destroy;
begin
  inherited Destroy;
  if not FCreated then exit;
  Close; FreeLibrary(FLibrairie);
end;

procedure TXDLLib.SendPercent;
begin
  if not FCreated then Exit;
  if not FPourcentP(FGetCurrentPercent) then ;
end;

procedure tXDLLib.Execute;
begin
  while not Terminated do
  begin
    FLocked := FIsLocked and (LockedID <> FID);
    if Assigned(FPourcentP) then
      Synchronize(SendPercent);
    Sleep(FRefresh);
  end;
end;

initialization
  Proc := nil;

end.
