unit uRegComp;

interface

uses Classes, PHRegistry, Windows;

type
  TLogEvent      = set of (leTimeSearch, leSizeFree, leDeleteFile, leWarning, leError);

  TPageAffichage = record
                     Detail,
                     Minimize       : boolean;
                     Refresh,
                     XPos, YPos     : Integer;
                   end;
  PPageAffichage = ^TPageAffichage;
  TPageNettoyage = record
                     CleanTrash,
                     CleanTemp,
                     CleanPerso,
                     Send2Trash,
                     Confirm,
                     Test           : boolean;
                     ROHConfirm     : Integer;
                   end;
  PPageNettoyage = ^TPageNettoyage;
  TPageJournal   = record
                     JournalActive,
                     EraseFile      : boolean;
                     LogEvent       : byte;
                     DirName        : string;
                   end;
  PPageJournal   = ^TPageJournal;
  TPageArbre     = record
                     FStream        : TMemoryStream;
                   end;
  PPageArbre     = ^TPageArbre;
  TPageService   = record
                     ServiceStart  : boolean;
                     IndexSearch,
                     FSearchSec,
                     FSearchSize   : Integer;
                   end;
  PPageService   = ^TPageService;

  TXDELRegistry  = class(TBaseRegistry)
  private
    FAppInstalled,
    FServiceInstalled : boolean;

    FPageAffichage    : PPageAffichage;
    FPageNettoyage    : PPageNettoyage;
    FPageJournal      : PPageJournal;
    FPageArbre        : PPageArbre;
    FPageService      : PPageService;
  protected
  public
    constructor Create;
    destructor Destroy; override;

    procedure Default; override;
    procedure SaveToRegistry; override;
    procedure LoadFromRegistry; override;

    property ApplicationInstalled: boolean read FAppInstalled;
    property ServiceInstalled    : boolean read FServiceInstalled;

    property Affichage : PPageAffichage read FPageAffichage;
    property Nettoyage : PPageNettoyage read FPageNettoyage;
    property Journal   : PPageJournal   read FPageJournal;
    property Arbre     : PPageArbre     read FPageArbre;
    property Service   : PPageService   read FPageService;
  end;

implementation

uses uWindows;

resourcestring
  rsAppInstalled     = 'Application';
  rsServiceInstalled = 'Service';

  rsAffichage      = 'Affichage';
    rsDetail       = 'Afficher le detail';
    rsReduce       = 'Mettre en icone';
    rsRefresh      = 'Vitesse de rafraichissement';
    rsXPos         = 'Position en X';
    rsYPos         = 'Position en Y';
  rsNettoyage      = 'Nettoyage';
    rsCleanTrash   = 'Vider la corbeille';
    rsCleanTemp    = 'Vider le dossier temporaire';
    rsPerso        = 'Recherche personnelle';
    rsSd2Trash     = 'Envoyer les elements dans la corbeille';
    rsConfirm      = 'Confirmer la suppression';
    rsTest         = 'Tester';
    rsROHConfirm   = 'Confirmer la suppression des fichiers RO et H';
  rsJournal        = 'Journal';
    rsActiveLog    = 'Activer la journalisation';
    rsEmplacement  = 'Emplacement du journal';
    rsEvenements   = 'Evenements';
    rsNouveauLog   = 'Nouveau journal  chaque recherche';
  rsPersonel       = 'Personnel';
    rsPersoSearch  = 'Recherche personnelle';
  rsService        = 'Service';
    rsStartService = 'Demarrer le service automatiquement';
    rsTypeClear    = 'Type de nettoyage';
    rsEveryTime    = 'Tous les ';
    rsMinFreeDsk   = 'Quand moins de ';

const
  dfAppInstalled     = false;
  dfServiceInstalled = false;
(******************************)
  dfDetail        = false;
  dfReduce        = true;
  dfRefresh       = 500;
(******************************)
  dfCleanTrash    = true;
  dfCleanTemp     = true;
  dfPerso         = true;
  dfSd2Trash      = true;
  dfConfirm       = true;
  dfTest          = false;
  dfROHConfirm    = 2;
(******************************)
  dfActiveLog     = true;
  dfEmplacement   = 'C:\';
  dfEvenements    = 30;       // <=> 11110 (binaire)
  dfNouveauLog    = true;
(******************************)
//  dfPersoSearch   = 0;
(******************************)
  dfStartService = true;
  dfTypeClear    = 1;
  dfEveryTime    = 10 * 60;
  dfMinFreeDsk   = 500 * 1024 * 1024; (* 500 Mo *)

constructor TXDELRegistry.Create;
begin
  New(FPageAffichage);
  New(FPageNettoyage);
  New(FPageJournal);
  New(FPageArbre);
  New(FPageService);

  FPageArbre.FStream := TMemoryStream.Create;
  inherited Create;
end;

destructor TXDELRegistry.Destroy;
begin
  FPageArbre.FStream.Free;

  Dispose(FPageAffichage);
  Dispose(FPageNettoyage);
  Dispose(FPageJournal);
  Dispose(FPageArbre);
  Dispose(FPageService);
  inherited Destroy;
end;

procedure TXDELRegistry.Default;
begin
  FPageAffichage.Detail := dfDetail;
  FPageAffichage.Minimize := dfReduce;
  FPageAffichage.Refresh := dfRefresh;

  FPageNettoyage.CleanTrash := dfCleanTrash;
  FPageNettoyage.CleanTemp  := dfCleanTemp;
  FPageNettoyage.CleanPerso := dfPerso;
  FPageNettoyage.Send2Trash := dfSd2Trash;
  FPageNettoyage.Confirm    := dfConfirm;
  FPageNettoyage.Test       := dfTest;
  FPageNettoyage.ROHConfirm := dfROHConfirm;

  FPageJournal.JournalActive:= dfActiveLog;
  FPageJournal.EraseFile    := dfNouveauLog;
  FPageJournal.LogEvent     := dfEvenements;
  FPageJournal.DirName      := dfEmplacement;

  FPageService.ServiceStart := dfStartService;
  FPageService.IndexSearch  := dfTypeClear;
  FPageService.FSearchSec   := dfEveryTime;
  FPageService.FSearchSize  := dfMinFreeDsk;

  FPageArbre.FStream.Clear;
end;

procedure TXDELRegistry.SaveToRegistry;
begin
  inherited SaveToRegistry;
  if Registry.OpenKey(getVersionKey + '\' + rsAffichage, true) then //
  try
    WriteValue(rsDetail, FPageAffichage.Detail, dfDetail);
    WriteValue(rsReduce, FPageAffichage.Minimize, dfReduce);
    WriteValue(rsRefresh, FPageAffichage.Refresh, dfRefresh);
    WriteValue(rsXPos, FPageAffichage.XPos, -1);
    WriteValue(rsYPos, FPageAffichage.YPos, -1);
  finally
    Registry.CloseKey;
  end;

  if Registry.OpenKey(getVersionKey + '\' + rsNettoyage, true) then //
  try
    WriteValue(rsCleanTrash, FPageNettoyage.CleanTrash, dfCleanTrash);
    WriteValue(rsCleanTemp, FPageNettoyage.CleanTemp, dfCleanTemp);
    WriteValue(rsPerso, FPageNettoyage.CleanPerso, dfPerso);
    WriteValue(rsSd2Trash, FPageNettoyage.Send2Trash, dfSd2Trash);
    WriteValue(rsConfirm, FPageNettoyage.Confirm, dfConfirm);
    WriteValue(rsTest, FPageNettoyage.Test, dfTest);
    WriteValue(rsROHConfirm, FPageNettoyage.ROHConfirm, dfROHConfirm);
  finally
    Registry.CloseKey;
  end;

  if Registry.OpenKey(getVersionKey + '\' + rsJournal, true) then //
  try
    WriteValue(rsActiveLog, FPageJournal.JournalActive, dfActiveLog);
    WriteValue(rsEmplacement, FPageJournal.DirName, dfEmplacement);
    WriteValue(rsEvenements, FPageJournal.LogEvent, dfEvenements);
    WriteValue(rsNouveauLog, FPageJournal.EraseFile, dfNouveauLog);
  finally
    Registry.CloseKey;
  end;

  if Registry.OpenKey(getVersionKey + '\' + rsService, true) then //
  try
    WriteValue(rsStartService, FPageService.ServiceStart, dfStartService);
    WriteValue(rsTypeClear, FPageService.IndexSearch, dfTypeClear);
    WriteValue(rsEveryTime, FPageService.FSearchSec, dfEveryTime);
    WriteValue(rsMinFreeDsk, FPageService.FSearchSize, dfMinFreeDsk);
  finally
    Registry.CloseKey;
  end;

  if Registry.OpenKey(getVersionKey + '\' + rsPersonel, true) then
  try
    if FPageArbre.FStream.Size > 0 then
      Registry.WriteBinaryData(rsPersoSearch, FPageArbre.FStream.Memory^, FPageArbre.FStream.Size)
    else
      Registry.DeleteValue(rsPersoSearch);
  finally
    Registry.CloseKey;
  end;
end;

procedure TXDELRegistry.LoadFromRegistry;
begin
  inherited LoadFromRegistry;
  Registry.OpenKey(getVersionKey, false);
  try
    FAppInstalled     := ReadValue(rsAppInstalled, dfAppInstalled);
    FServiceInstalled := ReadValue(rsServiceInstalled, dfServiceInstalled);
  finally
    Registry.CloseKey;
  end;

  Registry.OpenKey(getVersionKey + '\' + rsAffichage, false); //
  try
    FPageAffichage.Detail   := ReadValue(rsDetail, dfDetail);
    FPageAffichage.Minimize := ReadValue(rsReduce, dfReduce);
    FPageAffichage.Refresh  := ReadValue(rsRefresh, dfRefresh);
    FPageAffichage.XPos     := ReadValue(rsXPos, -1);
    FPageAffichage.YPos     := ReadValue(rsYPos, -1);
  finally
    Registry.CloseKey;
  end;

  Registry.OpenKey(getVersionKey + '\' + rsNettoyage, false); //
  try
    FPageNettoyage.CleanTrash := ReadValue(rsCleanTrash, dfCleanTrash);
    FPageNettoyage.CleanTemp  := ReadValue(rsCleanTemp, dfCleanTemp);
    FPageNettoyage.CleanPerso := ReadValue(rsPerso, dfPerso);
    FPageNettoyage.Send2Trash := ReadValue(rsSd2Trash, dfSd2Trash);
    FPageNettoyage.Confirm    := ReadValue(rsConfirm, dfConfirm);
    FPageNettoyage.Test       := ReadValue(rsTest, dfTest);
    FPageNettoyage.ROHConfirm := ReadValue(rsROHConfirm, dfROHConfirm);
  finally
    Registry.CloseKey;
  end;

  Registry.OpenKey(getVersionKey + '\' + rsJournal, false); //
  try
    FPageJournal.JournalActive := ReadValue(rsActiveLog, dfActiveLog);
    FPageJournal.EraseFile     := ReadValue(rsNouveauLog, dfNouveauLog);
    FPageJournal.LogEvent      := ReadValue(rsEvenements, dfEvenements);
    FPageJournal.DirName       := ReadValue(rsEmplacement, dfEmplacement);
  finally
    Registry.CloseKey;
  end;

  Registry.OpenKey(getVersionKey + '\' + rsService, false); //
  try
    FPageService.ServiceStart := ReadValue(rsStartService, dfStartService);
    FPageService.IndexSearch  := ReadValue(rsTypeClear, dfTypeClear);
    FPageService.FSearchSec   := ReadValue(rsEveryTime, dfEveryTime);
    FPageService.FSearchSize  := ReadValue(rsMinFreeDsk, dfMinFreeDsk);
  finally
    Registry.CloseKey;
  end;

  Registry.OpenKey(getVersionKey + '\' + rsPersonel, false); //
  try
    if Registry.GetDataSize(rsPersoSearch) > 0 then
    begin
      FPageArbre.FStream.SetSize(Registry.GetDataSize(rsPersoSearch));
      Registry.ReadBinaryData(rsPersoSearch, FPageArbre.FStream.Memory^, FPageArbre.FStream.Size);
    end;
  finally
    Registry.CloseKey;
  end;
end;

end.
